# -*-: big5
from mez_xml.tools import nez_web

class tb_rss(nez_web):
    def __init__(self, ofo):
        super(tb_rss, self).__init__()
        self.ofo = ofo
        pass
    
    def _root(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''''')
            cdata = data.setdefault('content', {})
            self.content(data, cdata)
            self.ofo.write('''''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    def content(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<response ezid="content">
 <error>0</error>
 <rss version="0.91">
  <channel>
   <title>''')
            odata = data.setdefault('title', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</title>
   <link>''')
            odata = data.setdefault('link', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</link>
   <description>''')
            odata = data.setdefault('description', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</description>
   <language>''')
            odata = data.setdefault('language', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</language>
   
   ''')
            cdata = data.setdefault('items', {})
            self.items(data, cdata)
            self.ofo.write('''
   
  </channel>
 </rss>
</response>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    def items(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<item ezid="items">
    <title>''')
            odata = data.setdefault('title', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</title>
    <link>''')
            odata = data.setdefault('link', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</link>
    <description>''')
            odata = data.setdefault('description', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</description>
   </item>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    pass





class rc4:
    def __init__(self, key):
	self.key = key
	self.sbox = sbox = list(range(256))
	klen = len(key)
	j = 0
	for i in range(256):
	    j = (j + sbox[i] + key[i % klen]) % 256
	    sbox[i], sbox[j] = sbox[j], sbox[i]
	    pass
	self.i = self.j = 0
	pass
    
    def encrypt(self, plain):
	i = self.i
	j = self.j
	sbox = self.sbox
	
	cipher = []
	for idx in range(len(plain)):
	    i = (i + 1) % 256
	    j = (j + sbox[i]) % 256
	    
	    sbox[i], sbox[j] = sbox[j], sbox[i]
	    
	    t = (sbox[i] + sbox[j]) % 256
	    
	    cipher.append(t ^ plain[idx])
	    pass
	
	self.i = i
	self.j = j
	return cipher
    pass


def get_key(doc_id):
    import md5, array, config
    m = md5.md5(config.GINGIN_COMMENT_SECRET + str(doc_id))
    key = array.array('B', m.digest())[:1]
    return key


def gen_an_attack(key):
    import random
    rc4_o = rc4(key)
    plain = [int(random.random() * 256) for i in range(32)]
    cipher = rc4_o.encrypt(plain)
    return plain, cipher

if __name__ == '__main__':
    key = get_key(257)
    plain, cipher = gen_an_attack(key)
    print key
    print plain
    print cipher
    rc4_o = rc4(key)
    print rc4_o.encrypt(cipher)
    pass



# -*-: big5
from mez_xml.tools import nez_web

class all_kws(nez_web):
    def __init__(self, ofo):
        super(all_kws, self).__init__()
        self.ofo = ofo
        pass
    
    def _root(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<div id="content">
<div class="title">
¦@¦³ ''')
            odata = data.setdefault('keywords_num', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write(''' ­ÓÃöÁä¦r:
</div>

<div class="text">
<div id="keywords">
''')
            cdata = data.setdefault('keywords', {})
            self.keywords(data, cdata)
            self.ofo.write('''
</div>
<script type="text/javascript" language="javascript">
<!--
function create_table(data) {
	var tb = document.createElement("table");
	var tr, td;
	var i, j;

	for(i in data) {
		tr = document.createElement("tr");
		for(j in data[i]) {
			td = document.createElement("td");
			td.appendChild(data[i][j]);
			tr.appendChild(td);
		}
		tb.appendChild(tr);
	}
	return tb;
}

function show_keywords() {
	var kws_o = document.getElementById("keywords");
	var kw_o, kws_p_o;
	var kws = [];
	var tb = [], tb_c;
	var tb_o, a_o, txt_o;
	var i, j;

	for(i = 0; i < kws_o.childNodes.length; i++) {
		kw_o = kws_o.childNodes[i];
		if(kw_o.tagName == "span" || kw_o.tagName == "SPAN") {
			kws.push(kw_o.childNodes[0].data);
		}
	}
	kws_p_o = kws_o.parentNode;
	kws_p_o.removeChild(kws_o);
	for(i = 0; i < kws.length; i += 6) {
		tb_c = [];
		for(j = 0; j < 6; j++) {
			if((i + j) >= kws.length) {
				txt_o = document.createTextNode("");
				tb_c.push(txt_o);
			} else {
				txt_o = document.createTextNode(kws[i + j]);
				a_o = document.createElement('a');
				a_o.setAttribute("href", "''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_comm(odata))
            self.ofo.write('''/show_kw_docs/" + kws[i + j]);
				a_o.appendChild(txt_o);
				tb_c.push(a_o);
			}
		}
		tb.push(tb_c);
	}
	tb_o = create_table(tb);
	kws_p_o.appendChild(tb_o);
}

init_funcs.push(show_keywords);
-->
</script>

</div>

</div>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    def keywords(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<span ezid="keywords">''')
            odata = data.setdefault('keyword', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</span>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    pass




#!/usr/local/bin/python
from GinGin import GinGin_CGI

if __name__ == '__main__':
    import fcgi
    import socket

    try:
        while fcgi.isFCGI():
            req = fcgi.Accept()
            GinGin_CGI.GinGin(req)
            req.Finish()
            pass
    except socket.error:
        pass
    pass




# -*-: big5
from mez_xml.tools import nez_web

class tb_view(nez_web):
    def __init__(self, ofo):
        super(tb_view, self).__init__()
        self.ofo = ofo
        pass
    
    def _root(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<html>
<head>
<link href="''')
            odata = data.setdefault('base_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/GinGin.css" rel="stylesheet"></link>
<meta value="text/html; charset=big5" http-equiv="Content-Type"></meta>
<script src="''')
            odata = data.setdefault('base_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/js/GinGin.js"></script>
</head>
''')
            cdata = data.setdefault('content', {})
            self.content(data, cdata)
            self.ofo.write('''
</html>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    def content(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<body ezid="content">
<div id="tb_view">
<table cellpadding="0" cellspacing="0" id="note">
<tr style="line-height: 0px">
<td><img src="''')
            odata = data.setdefault('base_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''images/green_box/luc.jpg"></img></td>
<td background="''')
            odata = data.setdefault('base_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''images/green_box/ub.jpg"></td>
<td><img src="''')
            odata = data.setdefault('base_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''images/green_box/ruc.jpg"></img></td>
</tr>
<tr>
<td style="background-repeat: repeat-y;" background="''')
            odata = data.setdefault('base_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''images/green_box/lb.jpg"></td>
<td width="100%">
   <div class="title">''')
            odata = data.setdefault('title', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</div>
   若是要引用此文，請用網址:
	<a href="''')
            odata = data.setdefault('base_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''GinGin_TB.py/trackback/''')
            odata = data.setdefault('doc_id', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''">
	''')
            odata = data.setdefault('base_url', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''GinGin_TB.py/trackback/''')
            odata = data.setdefault('doc_id', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''
	</a>
</td>
<td style="background-repeat: repeat-y;" background="''')
            odata = data.setdefault('base_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''images/green_box/rb.jpg"></td>
</tr>
<tr style="line-height: 0px">
<td>
<img src="''')
            odata = data.setdefault('base_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''images/green_box/lbc.jpg"></img></td>
<td style="background-repeat: repeat-x;" background="''')
            odata = data.setdefault('base_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''images/green_box/bb.jpg"></td>
<td><img src="''')
            odata = data.setdefault('base_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''images/green_box/rbc.jpg"></img></td>
</tr>
</table>

<!-- all trackbacks of the page -->
<p>
   引用文章 [<a href="''')
            odata = data.setdefault('link', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''">''')
            odata = data.setdefault('title', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</a>]
   所有網頁:
   <ul>
   ''')
            cdata = data.setdefault('pings', {})
            self.pings(data, cdata)
            self.ofo.write('''
   </ul>
   總共 ''')
            odata = data.setdefault('num_pings', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write(''' 筆
</p>
</div>
   
</body>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    def pings(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<li ezid="pings">
   <a href="''')
            odata = data.setdefault('url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''">''')
            odata = data.setdefault('title', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</a>
   </li>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    pass




#! env python

class NO_LOGIN_ERR:
    def __init__(self):
	pass


class REDIRECT:
    def __init__(self, url, msg):
        self.url = url
        self.msg = msg
	pass


# DONE: Use 'env' from octr instead of 'os.environ'.
class anony_user:
    def __init__(self):
	pass
    
    def get_login_user(self, octr):
	return 'guest'
    
    def get_id_from_user(self, act, name):
	return 'guest'
    
    def get_user_from_id(self, act, uid):
	return 'guest'
    
    def authorize(self, act, user, task, doc_id=-1):
	return 1
    
    def login(self, act, form):
	print 'Content-Type: text/html'
	print
	print 'Do not support login()'
    
    def show_login(self, act):
	print 'Content-Type: text/html'
	print
	print 'Do not support show_login()'


class HTTP_auth:
    def __init__(self):
	pass
    
    def get_login_user(self, octr):
	import os
	user = None
	try:
	    user = octr.env['REMOTE_USER']
	except:
	    pass
	return user
    
    def get_id_from_user(self, act, name):
	return name
    
    def get_user_from_id(self, act, uid):
	return uid
    
    def authorize(self, act, user, task, doc_id=-1):
	import os, GinGin_db

        # When a action need to be authorized before using it,
        # it is redirected to GinGin_CGI_s.py for authory.
        # GinGin_CGI_s.py is authorized with HTTP authorization
        # against a password file specified in .htaccess
        def auth_first():
            import tools
            base, spath = tools.get_server_url(act.octr.env)
            url = base + 'GinGin_CGI_s.py'
            if act.octr.env.has_key('PATH_INFO'):
                url = url + act.octr.env['PATH_INFO']
                pass
            if act.octr.env.has_key('QUERY_STRING') and \
                    act.octr.env['QUERY_STRING']:
                url = url + '?' + act.octr.env['QUERY_STRING']
                pass
	    raise REDIRECT(url, 'Authory your-self, first.')

	method = act.octr.env['REQUEST_METHOD']
        # TODO: improve performance of matching tasks.
	if (task in ('update', 'add_afile', 'del_afile',
                     'update_url', 'del_kw_urls')) and \
           (user == None or method != 'POST'):
            auth_first()
            pass
        if (task == 'show_all_unpublished') and \
           (user == None or method != 'GET'):
            auth_first()
            pass
        if (task in ('show_afiles', 'get_afile', 'show_id_doc',
                     'show_id_doc_src', 'edit_id_doc', 'trackback')) and \
           (method != 'GET' or
            ((user == None) and (not GinGin_db.is_doc_published(doc_id)))):
            auth_first()
            pass
	return 1
    
    def login(self, act, form):
	print 'Content-Type: text/html'
	print
	print 'Do not support login()'
    
    def show_login(self, act):
	import os
	print 'Content-Type: text/plain'
	print 'Status: 401 Authentication is required.'
	print 'WWW-Authenticate: Basic realm="By Invitation Only"'
	print
	print 'Error if you can see this document!'
	print 'Please ask administractor to start HTTP Authentication'
	print 'for posting to GinGin_CGI.py.'




# -*-: big5
from mez_xml.tools import nez_web

class gingin(nez_web):
    def __init__(self, ofo):
        super(gingin, self).__init__()
        self.ofo = ofo
        pass
    
    def _root(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<meta value="text/html; charset=big5" http-equiv="Content-Type"></meta>
<link href="''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/rssfeed" type="application/rss+xml" rel="alternate" title="''')
            odata = data.setdefault('site', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write(''' RSS Feed"></link>
<title>''')
            odata = data.setdefault('title', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</title>
<link href="''')
            odata = data.setdefault('base_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/GinGin.css" type="text/css" rel="stylesheet"></link>
''')
            cdata = data.setdefault('headcontent', {})
            self.headcontent(data, cdata)
            self.ofo.write('''
<script src="''')
            odata = data.setdefault('base_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/js/GinGin.js" type="text/javascript" language="javascript">
<![CDATA[
dummy = 1;
]]>
</script>
<script type="text/javascript" language="javascript">
<!--
var init_funcs = new Array();

function call_init() {
	var i;

	for(i in init_funcs) {
		init_funcs[i]();
	}
}
-->
</script>
</head>

<body onload="javascript: call_init()">
<table id="outer">
<tr>
<td>

<!-- LAST DOCUMENTS -->
<div id="hide_bar"><a href="''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/show_to_robot/">robot</a></div>
<div id="sidebar">
<div id="last_docs">
<div class="title">最新文章(''')
            odata = data.setdefault('last_docs_num', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write(''')</div>
<ul>
''')
            cdata = data.setdefault('last_docs', {})
            self.last_docs(data, cdata)
            self.ofo.write('''
</ul>
<a href="''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''">首頁</a><a href="''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/new_doc">新編</a><a href="''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/last_comments">最新留言</a><br></br>
<a href="''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/rssfeed">Entries RSS</a></div>

<!-- HOT KEYWORDS -->
<div id="hot_kws">
<div class="title">重要關鍵字(''')
            odata = data.setdefault('hot_kws_num', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write(''')</div>
<ul>
''')
            cdata = data.setdefault('hot_kws', {})
            self.hot_kws(data, cdata)
            self.ofo.write('''
</ul>
<a href="''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/show_all_kws">所有關鍵字</a><br></br>
<a href="''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/edit_new_url">新增 URL</a>
</div>

''')
            cdata = data.setdefault('non_public', {})
            self.non_public(data, cdata)
            self.ofo.write('''

<div>
''')
            cdata = data.setdefault('personal_info', {})
            self.personal_info(data, cdata)
            self.ofo.write('''
</div>
</div>

</td><td>

''')
            cdata = data.setdefault('content', {})
            self.content(data, cdata)
            self.ofo.write('''

</td>
</tr>
</table>

</body>
</html>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    def headcontent(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<link ezid="headcontent"></link>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    def last_docs(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<li ezid="last_docs"><a href="''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/show_id_doc/''')
            odata = data.setdefault('id', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''">''')
            odata = data.setdefault('title', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</a></li>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    def hot_kws(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<li ezid="hot_kws"><a href="''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/show_kw_docs/''')
            odata = data.setdefault('name', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''">''')
            odata = data.setdefault('name', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write(''' (''')
            odata = data.setdefault('num', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write(''')</a></li>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    def non_public(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<div ezid="non_public" id="non_public">
<div class="title">非公開</div>
<ul>
<li><a href="''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/show_all_unpublished">未發佈</a></li>
</ul>
</div>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    def personal_info(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<div ezid="personal_info"></div>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    def content(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<div ezid="content" id="content">
</div>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    pass




# -*-: big5
from mez_xml.tools import nez_web

class error_page(nez_web):
    def __init__(self, ofo):
        super(error_page, self).__init__()
        self.ofo = ofo
        pass
    
    def _root(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''''')
            cdata = data.setdefault('content', {})
            self.content(data, cdata)
            self.ofo.write('''''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    def content(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<html ezid="content">
<head>
	<title>''')
            odata = data.setdefault('host', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</title>
<style>
<!--
*	{
	font-size:	12pt;
}

.title {
        font-size:      18pt;
        font-weight:    700; 
}
.error {
        margin-left:    20px;
        border-style:   solid;
        border-width:   1px;
        border-color:   #005A9C;
        padding:        2px 5px 2px 5px;
	background-color:	#ffcc33;
	width:		80%;
}
-->
</style>
</head>
<body>
<div class="title">ERROR</div>
<div class="error">
<pre>
''')
            odata = data.setdefault('error', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''
</pre>
</div>
Please contact administrator!
</body>
</html>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    pass




# -*-: big5
from mez_xml.tools import nez_web

class last_comments(nez_web):
    def __init__(self, ofo):
        super(last_comments, self).__init__()
        self.ofo = ofo
        pass
    
    def _root(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<div id="content">

<div class="title">
Last Comments
</div>

<div class="text">

''')
            cdata = data.setdefault('comments', {})
            self.comments(data, cdata)
            self.ofo.write('''

</div>

</div>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    def comments(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<div ezid="comments" class="comment">
<div class="''')
            odata = data.setdefault('pdate', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''">on ''')
            odata = data.setdefault('pdate', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</div>
<span class="name"><a href="''')
            odata = data.setdefault('website', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''">''')
            odata = data.setdefault('name', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</a> said ..</span>
<div class="msg">''')
            odata = data.setdefault('msg', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</div>
<div class="control">
<a href="''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/edit_comment/''')
            odata = data.setdefault('doc_id', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''">reply</a>
<a href="''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/show_id_doc/''')
            odata = data.setdefault('doc_id', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''">goto</a>
</div>
</div>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    pass




# -*-: big5
from mez_xml.tools import nez_web

class unpublished(nez_web):
    def __init__(self, ofo):
        super(unpublished, self).__init__()
        self.ofo = ofo
        pass
    
    def _root(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<div id="content">

<div class="title">
未發佈文件:
</div>
<div class="text">
<table>
<tr><th>主題</th><th>作者</th><th>修改日期</th></tr>
  ''')
            cdata = data.setdefault('docs', {})
            self.docs(data, cdata)
            self.ofo.write('''
</table>
</div>

</div>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    def docs(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<tr ezid="docs">
    <td>
      <a href="''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/show_id_doc/''')
            odata = data.setdefault('doc_id', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''">
      ''')
            odata = data.setdefault('title', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''
      </a>
    </td>
    <td>
      ''')
            odata = data.setdefault('creator', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''
    </td>
    <td>
      ''')
            odata = data.setdefault('pdate', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''
    </td>
  </tr>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    pass




# -*-: big5
from mez_xml.tools import nez_web

class comment(nez_web):
    def __init__(self, ofo):
        super(comment, self).__init__()
        self.ofo = ofo
        pass
    
    def _root(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<div id="comment">

<div class="title">¯d¨¥: ¤å³¹ ''')
            odata = data.setdefault('doc_id', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</div>
<form action="''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''" onsubmit="javascript:return nospam()" method="get">
<input type="hidden" name="func" value="add_comment"></input>
<input type="hidden" name="doc_id" value="''')
            odata = data.setdefault('doc_id', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''"></input>
<input type="hidden" name="key" id="key"></input>
<table>
<tr><td><input accesskey="N" maxlength="60" type="text" name="name" size="20"></input>*Name(N)</td></tr>
<tr><td><input accesskey="M" maxlength="60" type="text" name="email" size="20"></input>*Mail(M)</td></tr>
<tr><td><input accesskey="W" maxlength="60" type="text" name="website" size="20"></input>Website(W)</td></tr>
<tr><td><textarea accesskey="C" rows="5" cols="40" id="msg" name="msg">comment...(C)</textarea></td></tr>
</table>
<input accesskey="S" type="submit" value="Save(S)"></input>
<span id="working">+</span>
</form>
<script src="''')
            odata = data.setdefault('base_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/js/rc4.js" language="Javascript">
var dummy;
</script>
<script language="Javascript">
var plain = "''')
            odata = data.setdefault('attack_plain', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''";
var cipher = "''')
            odata = data.setdefault('attack_cipher', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''";
</script>
<script src="''')
            odata = data.setdefault('base_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/js/base64.js" language="Javascript">
var dummy;
</script>
<script src="''')
            odata = data.setdefault('base_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/js/antispam.js" language="Javascript">
var dummy;
</script>

</div>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    pass




from string import join
import re

def escape_cdata(data):
    reo_xml_spec = re.compile('[&<>]')
    parts = []
    last_pos = 0
    mo = reo_xml_spec.search(data, last_pos)
    while mo != None:
	pos = mo.start()
	c = data[pos]
	if c == '&':
	    c = '&amp;'
	elif c == '<':
	    c = '&lt;'
	else:
	    c = '&gt;'
	parts.append(data[last_pos:pos] + c)
	last_pos = pos + 1
	mo = reo_xml_spec.search(data, last_pos)
    parts.append(data[last_pos:])
    return join(parts, '')

def escape_url(url):
    if not url:
	return 'http://invalid/empty/url'
    if not (url.startswith('http:') or url.startswith('HTTP:')):
	url = 'http://invalid/' + url
	pass
    url = url.replace('<', '%3C')
    url = url.replace('>', '%3E')
    url = url.replace('\'', '%27')
    url = url.replace('"', '%22')
    url = url.replace('\\', '%5C')
    url = url.replace(' ', '%20')
    return url

def trans_to_xml(data, tag='', lvl=0):
    from types import StringTypes, ListType, TupleType, SliceType, NoneType
    from types import MethodType, FunctionType, DictType
    indent = '  ' * lvl
    if type(data) in (ListType, TupleType, SliceType):
	items = map(lambda x: trans_to_xml(x, tag, lvl), data);
	retr = join(items, '')
    elif type(data) in (MethodType, FunctionType):
	retr = trans_to_xml(data(), tag, lvl)
    elif isinstance(data, DictType):
	sub_retr = map(lambda x:trans_to_xml(x[1], x[0], lvl + 1), data.items())
	sub_retr = join(sub_retr, '')
	retr = u'%s<%s>\n%s%s</%s>\n' % (indent, tag, sub_retr, indent, tag)
    elif isinstance(data, StringTypes):
	retr = u'%s<%s>%s</%s>\n' % (indent, tag, data, tag)
    else:
	retr = u'%s<%s>%s</%s>\n' % (indent, tag, str(data), tag)
    return retr


if __name__ == "__main__":
    from types import DictType
    d = {'a': 123, 'b': 'kkk', 'c': (1, 2, 3)}
    print type(d), DictType
    print trans_to_xml(d, 'test')



from config import GINGIN_ENCODING

def list_pair_to_pairs_list(l1, l2):
    assert(len(l1) == len(l2))
    return [(l1[i], l2[i]) for i in range(len(l1))]

def get_server_url(env):
    spath = 'http://' + env['SERVER_NAME']
    if env['SERVER_PORT'] != '80':
	spath = spath + ':' + env['SERVER_PORT']
    spath = spath + env['SCRIPT_NAME']
    last = spath.rfind('/')
    base = spath[:last+1]
    return base, spath

def mk_not_imp(name):
    def _not_imp(self, *args, **kws):
	raise NotImplementedError, '%s.%s()' % (self.__class__.__name__, name)
    return _not_imp

def hack_str(s):
    if isinstance(s, unicode):
	return s.encode(GINGIN_ENCODING)
    return str(s)

def redirect(ofo, url):
    ofo.write('Location: %s\n' % (url,))
    ofo.write('Content-Type: text/html; charset=' + GINGIN_ENCODING + '\n')
    ofo.write('\n\n<html><body><a href="%s">%s</a></body></html>\n' % (url, url))
    pass



# -*-: big5
from mez_xml.tools import nez_web

class edit(nez_web):
    def __init__(self, ofo):
        super(edit, self).__init__()
        self.ofo = ofo
        pass
    
    def _root(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<div class="content">
<div class="title">編輯</div>
<form action="''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''" method="post">
<input type="hidden" name="func" value="update"></input>
<input type="hidden" id="doc_id" value="''')
            odata = data.setdefault('doc_id', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''" name="doc_id"></input>
<table>
<tr><td>

<table>
<tr>
<td>標題(I)</td>
<td>
<input name="title" accesskey="I" value="''')
            odata = data.setdefault('title', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''" maxlength="80" type="text" id="title" size="60"></input>
</td>
</tr>
<tr>
<td>關鍵字(K)</td>
<td>
<input name="keywords" accesskey="K" maxlength="200" type="text" id="keywords" size="40"></input>
<input checked="yes" type="checkbox" name="update_pdate"></input>更新修改日期
<input checked="''')
            odata = data.setdefault('published', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''" type="checkbox" id="published" name="published"></input>發佈
<div id="kw_store">
''')
            cdata = data.setdefault('keywords', {})
            self.keywords(data, cdata)
            self.ofo.write('''
</div>
<script>
<!--
var kws_o = document.getElementById("keywords");
var kw_store = document.getElementById("kw_store");
var i, kws;
kws = "";
for(i = 0; i < kw_store.childNodes.length; i++) {
	child = kw_store.childNodes[i];
	if(child.tagName == "span" || child.tagName == "SPAN") {
		kws = kws + child.childNodes[0].data + ";";
	}
}
kws_o.setAttribute("value", kws);
kw_store.parentNode.removeChild(kw_store);
var published = document.getElementById("published");
if(published.getAttribute("checked") == "true")
	published.checked = true;
else
	published.checked = false;
-->
</script>
</td>
</tr>
</table>

</td></tr>
<tr><td>
(T)<br></br>   
<textarea accesskey="T" rows="20" cols="70" id="doc_body" name="doc_body">''')
            odata = data.setdefault('body', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</textarea>
</td></tr>
</table>
<input accesskey="S" type="submit" value="更新(S)"></input>
</form>
		
</div>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    def keywords(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<span ezid="keywords">''')
            odata = data.setdefault('keyword', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</span>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    pass




#!/usr/local/bin/python
import sys
import represent
import GinGin_acts
import tools
from config import *


# DONE: Attach an output file and environment variables on output_ctr.
class output_ctr:
    def __init__(self, out, env):
	self.mix= True
	self.media_type = 'text/html'
        self.charset = GINGIN_ENCODING
	self.ofo = out
        self.env = env
	pass
    
    def set_last_modified(self, tm):
	self.last_modified = tm
	pass
    
    def clear_last_modified(self):
	del self.last_modified
	pass
    
    def stop_mix(self):
	self.mix = False
	
    def set_media_type(self, mtype):
	self.media_type = mtype
	pass

    def set_charset(self, chset):
        self.charset = chset
        pass
    pass


def send_http_header(octr):
    from time import asctime, gmtime
    try:
	print >> octr.ofo, 'Last-Modified: ' + octr.last_modified
    except AttributeError:
	pass
    print >> octr.ofo, 'Content-Type: ' + octr.media_type + '; charset=' + GINGIN_ENCODING
    print >> octr.ofo
    pass


class page_framework(object):
    get_server_url = tools.mk_not_imp('get_server_url')
    
    def __init__(self, skel_page):
	super(page_framework, self).__init__()
	self._skel_page = skel_page
	pass
    
    def get_last_docs(self):
	import GinGin_db
	docs = GinGin_db.get_last_docs(GINGIN_MOST_RECENT_N)
	r = [{'id': doc[0], 'title': doc[1]} for doc in docs]
	return r
    
    def get_hot_keywords(self):
	import GinGin_db
	from xml_template import escape_cdata
	hot_kws = GinGin_db.get_hot_keywords(GINGIN_MOST_HOT_KWS)
	r  = [{'name': kw[0], 'num': kw[1]} for kw in hot_kws]
	return r
    
    def __call__(self, data, temp, octr, env):
        from templates import gingin
        
	base, spath = self.get_server_url(env)
	
	if hasattr(gingin.gingin, 'content'):
	    del gingin.gingin.content
	    pass
	if self._skel_page:
	    page = self._skel_page(octr.ofo)
	    page.content = temp._root
	else:
	    page = temp
	    pass
	temp.safe = True
	page.safe = True
	
	pdata = {'content': data}
	pdata['site'] = GINGIN_SITE
	pdata['base_url'] = base
	pdata['gingin_url'] = spath
	pdata['title'] = data['title']
	pdata['last_docs'] = self.get_last_docs()
	pdata['last_docs_num'] = len(pdata['last_docs'])
	pdata['hot_kws'] = self.get_hot_keywords()
	pdata['hot_kws_num'] = len(pdata['hot_kws'])
        user = GINGIN_USER.get_login_user(octr)
        if user == None:
            pdata['non_public'] = ''
            pass
	page.gen_doc(pdata)
	pass
    pass

def GinGin(req):

    # actin_out is the output of request action.
    # If it is None, the action had generated output page for them-self,
    # or action_out comprises data, template, and, maybe, page skelelton
    # waiting for rendering.  Here, we render the data for it.
    form = req.getFieldStorage()
    try:
        octr = output_ctr(req.out, req.env)
        GinGin_acts.str = tools.hack_str
	action_out = GinGin_acts.GinGin_run_action(form, req.env, octr)
    except:
	import traceback
	tb_str = traceback.format_exc()
	tb_str = tb_str.replace('<', '&lt;')
	tb_str = tb_str.replace('>', '&gt;')
	data, temp, page_skel = represent.error_page('EXCEPTION:\n' + tb_str, octr.ofo)
        send_http_header(octr)
        temp.gen_doc(data)
        sys.exit(0)
	
    if action_out != None:       # There is data waiting for rendering
        # Render the data.
        from templates import gingin
        
        # page_skel is outest skeleton of output page.
        # data is data being showed by output page.
        # temp is the template used to render data.
        #
        # The result of rendering is ovened with page_skel to create
        # output page.
        if len(action_out) == 2:
            data, temp = action_out
            page_skel = gingin.gingin
            gingin.str = tools.hack_str
        elif len(action_out) == 3:  # actin_out is with page_skel
            data, temp, page_skel = action_out
            pass
        pfwrk = page_framework(page_skel)
        pfwrk.get_server_url = tools.get_server_url
        send_http_header(octr)
        octr.ofo.write("<?xml version=\"1.0\" encoding=\"%s\"?>\n" % (octr.charset,))
        if octr.media_type == 'text/html':
            octr.ofo.write('<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">\n')
            pass
        pfwrk(data, temp, octr, req.env)
        #sys.exit(0)
        pass
    pass

if __name__ == '__main__':
    import fcgi

    while fcgi.isFCGI():
        req = fcgi.Accept()
        GinGin(req)
        req.Finish()
        pass
    pass



#!evn python
import sys
from config import *
from string import join
from html_template import *
import tools

def error_page(msg, ofo):
    from templates import error_page
    from xml_template import escape_cdata
    data = {}
    data['host'] = GINGIN_SITE
    data['error'] = msg
    data['title'] = ''
    error_page.str = tools.hack_str
    return data, error_page.error_page(ofo), None

def redirect(url, msg):
    print 'Status: 307 Temporary Redirect'
    print 'Location: %s' % (url)
    print 'Content-Type: text/html; charset=' + GINGIN_ENCODING
    print
    print '<html><body>%s</body><html>' % (msg)


def show_update_ok(doc_id, title, spath):
    from xml_template import escape_cdata
    title = title.decode('utf8').encode(GINGIN_ENCODING)
    print 'Location: %s?func=show_id_doc&doc_id=%d' % (spath, doc_id)
    print 'Content-Type: text/html; charset=' + GINGIN_ENCODING
    print
    print '<html><body>%s</body></html>' % (escape_cdata(title),)


def show_update_url_ok(url_id, url, abstract, title, keyword):
    from xml_template import escape_cdata
    keyword = escape_cdata(keyword.decode('utf8').encode(GINGIN_ENCODING))
    title = title.decode('utf8').encode(GINGIN_ENCODING)
    url = escape_cdata(url.decode('utf8').encode(GINGIN_ENCODING))
    print 'Location: GinGin_CGI_s.py?func=show_kw_urls&keyword=%s' % (keyword)
    print 'Content-Type: text/html; charset=' + GINGIN_ENCODING
    print
    print '<html><body>%s - %s</body></html>' % (title, url)


def show_del_kw_urls_ok(keyword, url_ids):
    from xml_template import escape_cdata
    keyword = keyword.decode('utf8').encode(GINGIN_ENCODING)
    print 'Location: GinGin_CGI_s.py/show_kw_urls/%s' % (keyword)
    print 'Content-Type: text/html; charset=' + GINGIN_ENCODING
    print
    print '<html><body>%s</body></html>' % (escape_cdata(keyword),)


def show_doc(kws, doc_id, title, doc_body, pdate, creator, comments=None):
    from xml_template import escape_cdata, escape_url
    kws = [escape_cdata(kw) for kw in kws]
    title = escape_cdata(title)
    doc_body = escape_cdata(doc_body)
    doc = {'id': doc_id, 'title': title, 'body': doc_body, 'date': pdate, 'creator': creator, 'keywords': {'key':kws}}
    out = {'doc': doc}
    if comments:
	comments = [{'cmt_id': c[0], 'name': escape_cdata(c[1]), 'email': escape_cdata(c[2]), 'website': escape_url(c[3]), 'msg': escape_cdata(c[4]), 'pdate': c[5]} for c in comments]
	doc['comments'] = {'comment': comments}
	pass
    return out


def show_doc_src(kws, doc_id, title, doc_body, pdate, creator):
    skws = join(kws, ';').encode(GINGIN_ENCODING)
    title = title.encode(GINGIN_ENCODING)
    doc_body = doc_body.encode(GINGIN_ENCODING)
    data = {'encoding': GINGIN_ENCODING, 'keywords': skws, 'doc_id': `doc_id`, 'title': title, 'doc_body': doc_body, 'pdate': pdate.encode(GINGIN_ENCODING), 'creator': creator.encode(GINGIN_ENCODING)}
    jdata = {}
    print 'Content-Type: text/plain; charset=' + GINGIN_ENCODING
    print
    format_template(GINGIN_SHOW_DOC_SRC, data, jdata)


def edit_doc(kws, doc_id, title, doc_body, pdate, creator):
    from xml_template import escape_cdata
    kws = [escape_cdata(kw) for kw in kws]
    title = escape_cdata(title)
    doc_body = escape_cdata(doc_body)
    out = {'edit': {'title': title, 'keywords': {'key': kws}, 'id': doc_id, 'body': doc_body, 'date': pdate, 'creator': creator}}
    return out


def show_docs_of_kw(keyword, docs, urls):
    from xml_template import escape_cdata, escape_url
    docs = map(lambda x: {'id': x[1], 'title': escape_cdata(x[2]), 'date': x[4], 'creator': x[5]}, docs) # doc_id, title, pdate, creator
    urls = map(lambda x: {'id': x[0], 'title': escape_cdata(x[1]), 'url': escape_url(x[2]), 'abstract': escape_cdata(x[3]), 'date': x[4], 'creator': x[5]}, urls)
    out = {}
    out['keyword'] = {'name': escape_cdata(keyword), 'docs': {'doc': docs}, 'urls': {'url': urls}}
    return out


def show_all_kws(kws):
    from xml_template import escape_cdata
    kws = [escape_cdata(kw) for kw in kws]
    out = {'keywords': {'key': kws}}
    return out


def show_afkeys_doc(doc_id, afkeys):
    out = {'doc': {'attaches': {'attach': afkeys}, 'id': doc_id}}
    return out
    

def show_deny(user, task, doc_id):
    print 'Content-Type: text/html; charset=' + GINGIN_ENCODING
    print
    print '<html><body>Permission Deny!</body></html>'
    return


def send_file(fname, asfname, mtype):
    from sys import stdout
    from os import stat
    from time import asctime, gmtime
    
    st = stat(fname)
    st_mtime = asctime(gmtime(st[8])) + ' GMT'
    print 'Content-Type: ' + mtype + '; name="' + asfname + '"'
    print 'Content-Disposition: attachment; filename="' + asfname + '"'
    print 'Last-Modified: ' + st_mtime
    print
    fo = open(fname, 'r')
    buf = fo.read(10240)
    while len(buf):
	stdout.write(buf)
	buf = fo.read(10240)
    return

def not_found(name):
    from xml_template import escape_cdata
    print 'Content-Type: text/plain'
    print 'Status: 404 not found'
    print
    print '404 Not found'
    print escape_cdata(name) + ' is not found!'
    print 'By GinGin.'
    return

def go_back(env):
    from xml_template import escape_url
    refer = env['HTTP_REFERER']
    refer = escape_url(refer)
    print 'Location: ' + refer
    print 'Content-Type: text/html; charset=' + GINGIN_ENCODING
    print
    print 'Move to ' + refer
    return

def show_edit_comment(doc_id, attack):
    plain = attack[1]
    cipher = attack[2]
    
    out = {'comment': {'doc_id': doc_id, 'attack': {'plain': plain, 'cipher': cipher}}}
    return out

def show_last_comments(comments):
    from tools import list_pair_to_pairs_list
    from xml_template import escape_cdata, escape_url
    comments = [(c[0], c[1], escape_cdata(c[2]), escape_cdata(c[3]), escape_url(c[4]), escape_cdata(c[5]), c[6]) for c in comments]
    fnames = ('cmt_id', 'doc_id', 'name', 'email', 'website', 'msg', 'pdate')
    d_comments = [dict(list_pair_to_pairs_list(fnames, comment)) for comment in comments]
    out = {'comments': {'comment': d_comments}}
    return out

def show_to_robot(docs, env):
    import tools
    base, spath = tools.get_server_url(env)
    print 'Content-Type: text/html'
    print
    print '<html><body>'
    for doc_id in docs:
	print '<a href="%s/show_id_doc/%d">%d</a>' % (spath, doc_id, doc_id)
    print '</body></html>'
    return

def show_rssfeed(abstracts):
    from xml_template import escape_cdata
    abstracts = map(lambda x: { 'doc_id': x[0], 'title': escape_cdata(x[1]), 'pdate': x[2], 'creator': x[3], 'abs': escape_cdata(x[4])}, abstracts)
    out = { 'feeds': { 'feed': abstracts}}
    return out

def show_kw_urls(keyword, urls):
    from xml_template import escape_cdata, escape_url
    keyword = escape_cdata(keyword)
    urls = map(lambda x: {'id': x[0], 'title': escape_cdata(x[1]), 'url': escape_url(x[2]), 'abstracts': [{'abstract': escape_cdata(x[3])}], 'date': x[4], 'creator': x[5]}, urls)
    out = { 'name': keyword, 'urls': urls, 'url_cnt': len(urls)}
    return out

def show_edit_url(url, kws):
    from xml_template import escape_cdata, escape_url
    kws = [escape_cdata(kw) for kw in kws]
    x = url
    url = {'id': url[0], 'title': escape_cdata(url[1]), 'url': escape_url(x[2]), 'abstract': escape_cdata(x[3]), 'date': x[4], 'creator': x[5]}
    url['keywords'] = {'key': kws}
    out = {'edit': {'url': url}}
    return out



# -*-: big5
from mez_xml.tools import nez_web

class doc(nez_web):
    def __init__(self, ofo):
        super(doc, self).__init__()
        self.ofo = ofo
        pass
    
    def _root(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<div id="content">
<div class="title">
''')
            odata = data.setdefault('title', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''

<div class="creator">
by ''')
            odata = data.setdefault('creator', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''
</div>

</div>
<div style="float: right" class="keyword">
<a href="javascript: init_mcol()">
2 Columns
</a>
</div>
<div class="keywords">
關鍵字:
<div class="keyword">
''')
            cdata = data.setdefault('keywords', {})
            self.keywords(data, cdata)
            self.ofo.write('''
</div>
</div>

<div class="rawtext" id="body">
<textarea id="rawbody">
''')
            odata = data.setdefault('text', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''
</textarea>
</div>

<div class="utime">
最後更新時間:
''')
            odata = data.setdefault('pdate', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write(''' |
<a href="''')
            odata = data.setdefault('base_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''GinGin_TB.py/trackback/''')
            odata = data.setdefault('doc_id', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''?__mod=view" onclick="open_trackback(this.href); return false;">
引用
</a>
</div>

<script src="''')
            odata = data.setdefault('base_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/js/mcol.js" type="text/javascript" language="javascript">
dummy = 1;
</script>
<script src="''')
            odata = data.setdefault('base_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/js/gindoc.js" type="text/javascript" language="javascript">
dummy = 1;
</script>
<script src="''')
            odata = data.setdefault('base_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/js/show_doc_body.js" type="text/javascript" language="javascript">
dummy = 1;
</script>
<script src="''')
            odata = data.setdefault('base_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/js/init_mcol.js" type="text/javascript" language="javascript">
dummy = 1;
</script>
<script>
<!--
var gingin_url = "''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_comm(odata))
            self.ofo.write('''";
var doc_id = ''')
            odata = data.setdefault('doc_id', {})
            self.ofo.write(self._esc_comm(odata))
            self.ofo.write(''';
var rawbody = document.getElementById("rawbody");
var body = document.getElementById("body");
rawbody.value = rawbody.value.replace("&", "&amp;", "g");
rawbody.value = rawbody.value.replace("<", "&lt;", "g");
rawbody.value = rawbody.value.replace(">", "&gt;", "g");
-->
</script>

<div id="func_bar">
<input accesskey="E" type="button" onclick="edit()" value="編輯(E)"></input>
<input accesskey="N" type="button" onclick="new_doc()" value="新編(N)"></input>
<input accesskey="A" type="button" onclick="show_afiles()" value="附加(A)"></input>
<input accesskey="C" type="button" onclick="show_edit_comment()" value="留言(C)"></input>
<input type="button" onclick="all_kws()" value="所有關鍵字"></input>
查詢:<input onchange="query(this.value)" type="text" value="關鍵字"></input>
</div>

<div id="comments">
COMMENTS:
''')
            cdata = data.setdefault('comment', {})
            self.comment(data, cdata)
            self.ofo.write('''
</div>

<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:trackback="http://madskills.com/public/xml/rss/module/trackback/" xmlns:dc="http://purl.org/dc/elements/1.1/">
    <rdf:Description trackback:ping="''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/trackback/''')
            odata = data.setdefault('doc_id', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''" dc:title="''')
            odata = data.setdefault('site', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write(''': ''')
            odata = data.setdefault('title', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''" rdf:about="''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/show_id_doc/''')
            odata = data.setdefault('doc_id', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''" dc:identifier="''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/show_id_doc/''')
            odata = data.setdefault('doc_id', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''"></rdf:Description>
</rdf:RDF>

</div>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    def keywords(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<a ezid="keywords" href="''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/show_kw_docs/''')
            odata = data.setdefault('keyword', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''">
''')
            odata = data.setdefault('keyword', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''
</a>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    def comment(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<div ezid="comment" class="comment">
<div class="pdate">on ''')
            odata = data.setdefault('pdate', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</div>
<span class="name"><a href="''')
            odata = data.setdefault('website', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''">''')
            odata = data.setdefault('name', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</a> said ..</span>
<div class="msg">
''')
            odata = data.setdefault('msg', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''
</div>
</div>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    pass




# -*-: big5
from mez_xml.tools import nez_web

class keyword(nez_web):
    def __init__(self, ofo):
        super(keyword, self).__init__()
        self.ofo = ofo
        pass
    
    def _root(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<div id="content">

<div class="title">
關鍵字 ''')
            odata = data.setdefault('name', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''
</div>
相關文件:

<div class="text">
<table>
<tr><th>主題</th><th>作者</th><th>修改日期</th></tr>
  ''')
            cdata = data.setdefault('docs', {})
            self.docs(data, cdata)
            self.ofo.write('''
</table>
</div>
<br></br>
外部 URL:

<div class="text">
<form action="''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''" method="post">
<dl>
''')
            cdata = data.setdefault('urls', {})
            self.urls(data, cdata)
            self.ofo.write('''
</dl>
<input type="hidden" name="func" value="del_kw_urls"></input>
<input type="hidden" name="keyword" value="''')
            odata = data.setdefault('name', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''"></input>
<input type="submit" value="刪除"></input>
</form>
</div>

</div>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    def docs(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<tr ezid="docs">
    <td>
      <a href="''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/show_id_doc/''')
            odata = data.setdefault('doc_id', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''">
      ''')
            odata = data.setdefault('title', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''
      </a>
    </td>
    <td>
      ''')
            odata = data.setdefault('creator', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''
    </td>
    <td>
      ''')
            odata = data.setdefault('pdate', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''
    </td>
  </tr>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    def urls(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<dt ezid="urls">
<input type="checkbox" name="url_id" value="''')
            odata = data.setdefault('url_id', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''"></input>
<a href="''')
            odata = data.setdefault('url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''">''')
            odata = data.setdefault('title', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</a>
- ''')
            odata = data.setdefault('creator', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''
<a href="''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/edit_url/''')
            odata = data.setdefault('url_id', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''">[edit]</a>
<dd>
''')
            odata = data.setdefault('abstract', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''
</dd>
</dt>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    pass




import sys
from string import join

def trans_to_javascript(data):
    from types import StringType, ListType, TupleType, SliceType, NoneType
    from types import MethodType, FunctionType
    if type(data) == StringType:
	jstr = `data.decode('utf8')`[1:]
    elif type(data) == ListType or type(data) == TupleType or type(data) == SliceType:
	data = map(lambda x: trans_to_javascript(x), data)
	jstr = 'Array(' + join(data, ', ') + ')'
    elif type(data) == NoneType:
	jstr = 'null'
    elif isinstance(data, MethodType) or isinstance(data, FunctionType):
	jstr = data()
    else:
	jstr = `data`
    return jstr


def format_str_template(istr, data, jdata):
    ostr = istr
    for d in data.items():
	key = d[0]
	value = d[1]
	rep = '$' + key + '$'
	skip = len(rep)
	pos = ostr.find(rep)
	while pos >= 0:
	    attach = None
	    rvalue = value
	    try:
		#
		# Is it in form '$key${{{ ...... }}}'?
		#
		if ostr[pos:pos + len(rep)+3] == (rep + '{{{'):
		    epos = ostr.find('}}}', pos)
		    if epos > 0:
			apos = pos + len(rep) + 3
			attach = ostr[apos:epos]
			try:
			    rvalue = value(attach)
			except TypeError, e:
			    # not a callable
			    rvalue = 'error'
			    pass
			skip = epos + 3 - pos
	    except IndexError:
		pass
	    ostr = ostr[:pos] + rvalue + ostr[pos + skip:]
	    pos = ostr.find(rep, pos + skip)
    for d in jdata.items():
	key = d[0]
	value = d[1]
	rep = '$' + key + '$'
	pos = ostr.find(rep)
	while pos >= 0:
	    repstr = trans_to_javascript(value)
	    ostr = ostr[:pos] + repstr + ostr[pos + len(rep):]
	    pos = ostr.find(rep, pos + len(rep))
    return ostr


def format_template(fn, data, jdata):
    fo = open(fn, 'r')
    istr = fo.read()
    fo.close()
    
    sys.stdout.write(format_str_template(istr, data, jdata))
	



# -*-: big5
from mez_xml.tools import nez_web

class kw_urls(nez_web):
    def __init__(self, ofo):
        super(kw_urls, self).__init__()
        self.ofo = ofo
        pass
    
    def _root(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<div class="content">
<div class="title">
關鍵字 ''')
            odata = data.setdefault('name', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''
共有 ''')
            odata = data.setdefault('url_cnt', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write(''' 個 URL:
</div>

<div class="text">
<form action="''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''" method="post">
<dl>
''')
            cdata = data.setdefault('urls', {})
            self.urls(data, cdata)
            self.ofo.write('''
</dl>
<input type="hidden" name="func" value="del_kw_urls"></input>
<input type="hidden" name="keyword" value="''')
            odata = data.setdefault('name', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''"></input>
<input type="submit" value="刪除"></input>
</form>
</div>
</div>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    def urls(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<dt ezid="urls">
<input type="checkbox" name="url_id" value="''')
            odata = data.setdefault('id', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''"></input>
<a href="''')
            odata = data.setdefault('url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''">''')
            odata = data.setdefault('title', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</a>
- ''')
            odata = data.setdefault('creator', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''
<a href="''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/edit_url/''')
            odata = data.setdefault('id', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''">[edit]</a>
''')
            cdata = data.setdefault('abstracts', {})
            self.abstracts(data, cdata)
            self.ofo.write('''
</dt>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    def abstracts(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<dd ezid="abstracts">
''')
            odata = data.setdefault('abstract', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''
</dd>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    pass




#!env python
import sqlite3
import string
from config import *
from time import strftime
from sets import Set

cx = None
_WORDCHARS = string.letters + string.digits + '-_'

def now():
    return strftime('%Y-%m-%d %H:%M:%S %Z')

class GG_DB_ERR:
    def __init__(self):
	pass

def update_doc(doc_id, title, keywords, context, uid, update_pdate, published):
    global cx
    if not cx:
	cx = sqlite3.connect(GINGIN_DB)
    cu = cx.cursor()
    if doc_id < 0:
	#
	# add a new doc
	#
	cmd = 'insert into docs(title, doc_body, pdate, creator, published) values(?, ?, ?, ?, ?)'
	cu.execute(cmd, (title, context, now(), uid, published and 1 or 0))
	cu.execute('select max(doc_id) from docs')
	doc_id = int(cu.fetchone()[0])
	del_set = Set()
	add_set = Set(keywords)
    else:
	#
	# update a doc
	#
	# choice keyword sets that should be add or del
	cmd = 'select a.keyword from keywords a inner join kws_docs b on a.kw_id = b.kw_id where b.doc_id = ?'
	cu.execute(cmd, (doc_id,))
	okws = cu.fetchall()
	okws = map(lambda x: x[0], okws)
	old_set = Set(okws)
	new_set = Set(keywords)
	del_set = old_set - new_set
	add_set = new_set - old_set
	# update
	if update_pdate:
	    cmd = 'update docs set title = ?, doc_body = ?, pdate = ?, creator = ?, published = ? where doc_id = ?'
	    cu.execute(cmd, (title, context, now(), uid,
                             published and 1 or 0, doc_id))
	else:
	    cmd = 'update docs set title = ?, doc_body = ?, creator = ?, published = ? where doc_id = ?'
	    cu.execute(cmd, (title, context, uid,
                             published and 1 or 0, doc_id))
	    pass
	pass
    # remove keywords
    cmd = 'delete from kws_docs where kw_id = (select kw_id from keywords where keyword = ?) and doc_id = ?'
    try:
	while 1:
	    kw = del_set.pop()
	    if not kw:
		break
	    try:
		cu.execute(cmd, (kw, doc_id))
	    except sqlite3.DatabaseError:
		pass
    except KeyError:
	pass
    # add kws_docs
    cmd = 'insert into kws_docs select * from (select kw_id, ? from keywords where keyword = ?)'
    try:
	while 1:
	    kw = add_set.pop()
	    if not kw:
		break
	    try:
		# add keywords
		cu.execute('insert into keywords(keyword) values(?)', (kw,))
	    except sqlite3.DatabaseError:
		pass
	    cu.execute(cmd, (doc_id, kw))
    except KeyError:
	pass
    # remove useless keywords (no doc reference by it in kws_docs)
    cu.execute('delete from keywords where kw_id in (select a.kw_id from keywords a left join kws_docs b on a.kw_id = b.kw_id left join kws_urls c on a.kw_id = c.kw_id where (b.kw_id isnull) and (c.kw_id isnull))')
    cx.commit()
    return doc_id


def get_kw_docs(keyword):
    global cx
    if not cx:
	cx = sqlite3.connect(GINGIN_DB)
    cu = cx.cursor()
    # get doc with specified keyword
    cmd = 'select c.doc_id, c.title, c.pdate, c.creator from keywords as a inner join kws_docs as b on a.kw_id = b.kw_id inner join docs as c on b.doc_id = c.doc_id where a.keyword = ? and c.published = 1'
    cu.execute(cmd, (keyword,))
    docs = cu.fetchall()
    cx.commit()
    return docs


def get_kw_urls(keyword):
    global cx
    if not cx:
	cx = sqlite3.connect(GINGIN_DB)
    cu = cx.cursor()
    
    cmd = 'select a.url_id, title, url, abstract, pdate, creator from urls a join kws_urls b on a.url_id = b.url_id join keywords c on b.kw_id = c.kw_id where c.keyword = ?'
    cu.execute(cmd, (keyword,))
    urls = cu.fetchall()
    res = map(lambda x: (x[0], x[1], x[2], x[3], x[4], x[5]), urls)
    cx.commit()
    return res

def get_url(url_id):
    global cx
    if not cx:
	cx = sqlite3.connect(GINGIN_DB)
    cu = cx.cursor()
    
    cmd = 'select url_id, title, url, abstract, pdate, creator from urls where url_id = ?'
    cu.execute(cmd, (url_id,))
    url = cu.fetchone()
    url = map(lambda x: x, url) 
    
    cmd = 'select keyword from keywords a join kws_urls b on a.kw_id = b.kw_id where b.url_id = ?'
    cu.execute(cmd, (url_id,))
    kws = cu.fetchall()
    kws = map(lambda x: x[0], kws)

    cx.commit()
    return (url, kws)

def update_url(url_id, title, url, abstract, uid, kws):
    global cx
    if not cx:
	cx = sqlite3.connect(GINGIN_DB)
    cu = cx.cursor()
    
    if url_id < 0:
	cmd = 'insert into urls select max(url_id) + 1, ?, ?, ?, ?, ? from urls'
	cu.execute(cmd, (title, url, abstract, now(), uid))
	cmd = 'select max(url_id) from urls'
	cu.execute(cmd)
	url_id = int(cu.fetchone()[0])
	del_set = Set()
	add_set = Set(kws)
    else:
	cmd = 'update urls set title = ?, url = ?, abstract = ?, pdate = ?, creator = ? where url_id = ?'
	cu.execute(cmd, (title, url, abstract, now(), uid, url_id))
	cmd = 'select keyword from keywords a join kws_urls b on a.kw_id = b.kw_id where b.url_id = ?'
	cu.execute(cmd, (url_id,))
	old_kws = map(lambda x: x[0], cu.fetchall())
	old_set = Set(old_kws)
	new_set = Set(kws)
	del_set = old_set - new_set
	add_set = new_set - old_set
    # remove kws_urls being removed by the user
    cmd = 'delete from kws_urls where kw_id = (select kw_id from keywords where keyword = ?) and url_id = ?'
    try:
	while 1:
	    kw = del_set.pop()
	    if not kw:
		break
	    cu.execute(cmd, (kw, url_id))
    except KeyError:
	pass
    # add kws_urls being add by tye user
    cmd = 'insert into kws_urls select kw_id, ? from keywords where keyword = ?'
    try:
	while 1:
	    kw = add_set.pop()
	    if not kw:
		break
	    try:
		cu.execute('insert into keywords select max(kw_id) + 1, ? from keywords', (kw,))
	    except sqlite3.DatabaseError:
		pass
	    cu.execute(cmd, (url_id, kw))
    except KeyError:
	pass
    # remove useless keywords (no doc reference by it in kws_docs)
    cu.execute('delete from keywords where kw_id in (select a.kw_id from keywords a left join kws_docs b on a.kw_id = b.kw_id left join kws_urls c on a.kw_id = c.kw_id where (b.kw_id isnull) and (c.kw_id isnull))')
    cx.commit()
    return url_id

def del_kw_urls(keyword, url_ids):
    from string import join
    global cx
    if not cx:
	cx = sqlite3.connect(GINGIN_DB)
    cu = cx.cursor()
    
    url_str = join(map(lambda x: str(x), url_ids), ', ')
    cmd = 'delete from kws_urls where url_id in (' + url_str + ') and kw_id in (select kw_id from keywords where keyword = ?)'
    cu.execute(cmd, (keyword,))
    # remove useless keywords (no doc reference by it in kws_docs)
    cu.execute('delete from keywords where kw_id in (select a.kw_id from keywords a left join kws_docs b on a.kw_id = b.kw_id left join kws_urls c on a.kw_id = c.kw_id where (b.kw_id isnull) and (c.kw_id isnull))')
    # remove useless urls (no keyword reference it any more)
    cu.execute('delete from urls where url_id in (select a.url_id from urls a left join kws_urls b on a.url_id = b.url_id where b.kw_id isnull)')
    cx.commit()
    return

def get_last_docs(n):
    global cx
    if not cx:
	cx = sqlite3.connect(GINGIN_DB)
    cu = cx.cursor()
    cmd = 'select doc_id, title, pdate, creator from docs where published = 1 order by pdate desc limit ?'
    cu.execute(cmd, (n,))
    docs = cu.fetchall()
    res = map(lambda x: (x[0], x[1], x[2], x[3]), docs)
    cx.commit()
    return res

def get_unpublished_docs():
    global cx
    if not cx:
	cx = sqlite3.connect(GINGIN_DB)
    cu = cx.cursor()
    cmd = 'select doc_id, title, pdate, creator from docs where published = 0 order by pdate desc'
    cu.execute(cmd)
    docs = cu.fetchall()
    res = map(lambda x: (x[0], x[1], x[2], x[3]), docs)
    cx.commit()
    return res

def get_last_doc_ids(n):
    global cx
    if not cx:
	cx = sqlite3.connect(GINGIN_DB)
    cu = cx.cursor()
    cmd = 'select doc_id from docs where published = 1 order by doc_id desc limit ?'
    cu.execute(cmd, (n,))
    docs = cu.fetchall()
    res = map(lambda x: x[0], docs)
    cx.commit()
    return res

def abstract_document(doc):
    if len(doc) > GINGIN_ABSTRACT_SZ:
	sz = GINGIN_ABSTRACT_SZ
	while sz < len(doc):
	    if doc[sz] in string.whitespace:
		break
	    sz = sz + 1
	    pass
	doc = doc[:sz] + ' ...'
	pass
    return doc


def get_last_abstracts(n):
    global cx
    if not cx:
	cx = sqlite3.connect(GINGIN_DB)
    cu = cx.cursor()
    cmd = 'select doc_id, title, pdate, creator, doc_body from docs where published = 1 order by pdate desc limit ?'
    cu.execute(cmd, (n,))
    docs = cu.fetchall()
    res = map(lambda x: (x[0], x[1], x[2], x[3], abstract_document(x[4])), docs)
    cx.commit()
    return res


def get_id_doc(doc_id):
    global cx
    if not cx:
	cx = sqlite3.connect(GINGIN_DB)
    cu = cx.cursor()
    # get doc
    cmd = 'select * from docs where doc_id = ?'
    cu.execute(cmd, (doc_id,))
    row = cu.fetchone()
    if not row:
        raise ValueError, \
            'The document, with id #%d, is not existed.' % (doc_id)
    title = row[1]
    doc_body = row[2]
    pdate = row[3]
    creator = row[4]
    published = row[5] and True or False
    # get keywords
    cmd = 'select keyword from keywords a inner join kws_docs b on a.kw_id = b.kw_id where b.doc_id = ?'
    cu.execute(cmd, (doc_id,))
    rows = cu.fetchall()
    kws = map(lambda x: x[0], rows)
    cx.commit()
    return (kws, doc_id, title, doc_body, pdate, creator, published)


def get_id_doc_attack(doc_id, cu=None):
    global cx
    wcu = cu
    if wcu == None:
	if not cx:
	    cx = sqlite3.connect(GINGIN_DB)
	    pass
	wcu = cx.cursor()
	pass
    cmd = 'select key, plain, cipher from antispam where doc_id = ?'
    wcu.execute(cmd, (doc_id,))
    row = wcu.fetchone()
    if row:
	attack = row[0], row[1], row[2]
    else:
	import antispam, array, base64
	key = antispam.get_key(doc_id)
	plain, cipher = antispam.gen_an_attack(key)
	key = base64.b64encode(array.array('B', key).tostring())
	plain = base64.b64encode(array.array('B', plain).tostring())
	cipher = base64.b64encode(array.array('B', cipher).tostring())
	cmd = 'insert into antispam(doc_id, key, plain, cipher) values(?, ?, ?, ?)'
	wcu.execute(cmd, (doc_id, key, plain, cipher))
	attack = key, plain, cipher
        pass
    
    if cu == None:
        cx.commit()
        pass
    return attack


def get_id_doc_comments(doc_id, cu=None):
    global cx
    wcu = cu
    if wcu == None:
	if not cx:
	    cx = sqlite3.connect(GINGIN_DB)
	    pass
	wcu = cx.cursor()
	pass
    cmd = 'select cmt_id, name, email, website, msg, pdate from comments where doc_id = ?'
    wcu.execute(cmd, (doc_id,))
    rows = wcu.fetchall()
    comments = [(row[0], row[1], row[2], row[3], row[4], row[5]) for row in rows]
    if cu == None:
        cx.commit()
        pass
    return comments


def add_id_doc_comment(doc_id, name, email, website, msg, key, cu=None):
    global cx
    wcu = cu
    if wcu == None:
	if not cx:
	    cx = sqlite3.connect(GINGIN_DB)
	    pass
	wcu = cx.cursor()
	pass
    cmd = 'select * from antispam where doc_id = ? and key = ?'
    wcu.execute(cmd, (doc_id, key))
    row = wcu.fetchone()
    if not row:
	return
    if website:
	cmd = 'insert into comments (doc_id, name, email, website, msg, pdate) values(?, ?, ?, ?, ?, ?)'
	wcu.execute(cmd, (doc_id, name, email, website, msg, now()))
    else:
	cmd = 'insert into comments (doc_id, name, email, msg, pdate) values(?, ?, ?, ?, ?)'
	wcu.execute(cmd, (doc_id, name, email, msg, now()))
	pass
    if cu == None:
	cx.commit()
    pass


def get_last_comments(num=5):
    global cx
    if not cx:
	cx = sqlite3.connect(GINGIN_DB)
    cu = cx.cursor()
    
    cmd = 'select cmt_id, doc_id, name, email, website, msg, pdate from comments order by cmt_id desc limit ?'
    cu.execute(cmd, (num,))
    rows = cu.fetchall()
    comments = [tuple(row) for row in rows]
    return comments


def get_all_kws(cu=None):
    global cx
    wcu = cu
    if wcu == None:
	if not cx:
	    cx = sqlite3.connect(GINGIN_DB)
	wcu = cx.cursor()
    wcu.execute('select keyword from keywords order by keyword')
    rows = wcu.fetchall()
    if cu == None:
	cx.commit()
    kws = map(lambda x:x[0], rows)
    return kws

def _range_cmp(x, y):
    if x[0] > y[1]:
	r = 1
    elif x[1] < y[0]:
	r = -1
    else:
	r = 0
    return r

def replace_kws(msg, kws):
    import re
    
    c = lambda x, y: len(x) - len(y);
    kws.sort(c, lambda x: x, True)
    kws = map(lambda x: re.escape(x), kws)
    kw_pattern = '\\$|([hH][tT][tT][pP]|[aA][tT][tT][aA][cC][hH]):[^ ]*'
    if kws:
	kw_pattern = kw_pattern + '|' + string.join(kws, '|')
	pass
    reo_kws = re.compile(kw_pattern)
    reo_url = re.compile('([hH][tT][tT][pP]|[aA][tT][tT][aA][cC][hH]):[^ ]*')
    
    mos = reo_kws.finditer(msg)
    
    def mark(mo):
	kw = mark.msg[mo.start():mo.end()]
	if reo_url.match(kw):
	    res = mark.msg[mark.last:mo.end()]
	elif kw == '$':
	    res = mark.msg[mark.last:mo.end()] + '$'
	elif (kw[0] not in mark.en_chars) or \
	((mark.msg[mo.start()-1] not in mark.en_chars) and \
	(mark.msg[mo.end()] not in mark.en_chars)):
	    res = mark.msg[mark.last:mo.start()] + '$' + kw + '$'
	else:
	    res = msg[mark.last:mo.end()]
	mark.last = mo.end()
	return res
    # setup closures
    mark.last = 0
    mark.msg = msg
    mark.en_chars = string.ascii_letters + string.digits
    
    substrs = map(mark, mos)
    return string.join(substrs,'') + msg[mark.last:]


def exist_doc_id(doc_id, cu=None):
    global cx
    wcu = cu
    if wcu == None:
	if not cx:
	    cx = sqlite3.connect(GINGIN_DB)
	wcu = cx.cursor()
    wcu.execute('select count(*) from docs where doc_id = ?', (doc_id,))
    cnt = wcu.fetchone()[0]
    if cu == None:
	cx.commit()
    return cnt > 0


def get_afname_mtype_by_afkey(doc_id, afkey, cu=None):
    global cx
    wcu = cu
    if wcu == None:
	if not cx:
	    cx = sqlite3.connect(GINGIN_DB)
	wcu = cx.cursor()
        pass

    cmd = 'select afname, mtype from afiles where doc_id = ? and afkey = ?'
    wcu.execute(cmd, (doc_id, afkey))
    rows = wcu.fetchall()
    if len(rows) == 0:
	return None, None
    return rows[0][0], rows[0][1]


def __mk_afname(doc_id, afkey, mtype, cu=None):
    global cx
    wcu = cu
    if wcu == None:
	if not cx:
	    cx = sqlite3.connect(GINGIN_DB)
	wcu = cx.cursor()
    
    wcu.execute('select max(af_id) + 1 from afiles')
    af_id = wcu.fetchone()[0]
    if af_id == None:
	af_id = 1
    
    afname = 'attaches/af-' + `doc_id` + '-' + `af_id`
    cmd = 'insert into afiles values(?, ?, ?, ?, ?)'
    wcu.execute(cmd, (af_id, doc_id, afkey, afname, mtype))
    
    if cu == None:
	cx.commit()
    return afname

def add_afile_to_doc(doc_id, afkey, fobj, mtype):
    global cx
    if not cx:
	cx = sqlite3.connect(GINGIN_DB)
    cu = cx.cursor()
    
    # get afname
    if not exist_doc_id(doc_id):
	raise GG_DB_ERR()
    afname, o_mtype = get_afname_mtype_by_afkey(doc_id, afkey, cu)
    if afname == None:
	afname = __mk_afname(doc_id, afkey, mtype, cu)
    elif o_mtype != mtype:
	cmd = 'update afiles set mtype = ? where doc_id = ? and afkey = ?'
	cu.execute(cmd, (mtype, doc_id, afkey))
    
    cx.commit()
    
    # save attach file
    afo = open(afname, 'w+')
    buf = fobj.read(10240)
    while len(buf):
	afo.write(buf)
	buf = fobj.read(10240)
    afo.close()
    return


def remove_afile_by_afkey(doc_id, afkey, cu=None):
    global cx
    wcu = cu
    if wcu == None:
	if not cx:
	    cx = sqlite3.connect(GINGIN_DB)
	wcu = cx.cursor()
    
    afname, mtype = get_afname_mtype_by_afkey(doc_id, afkey, wcu)
    if afname == None:
	raise GG_DB_ERR()
    
    cmd = 'delete from afiles where doc_id = ? and afkey = ?'
    wcu.execute(cmd, (doc_id, afkey))
    import os
    os.remove(afname)
    
    if cu == None:
	cx.commit()
    return

def is_doc_published(doc_id, cu=None):
    global cx
    wcu = cu
    if wcu == None:
	if not cx:
	    cx = sqlite3.connect(GINGIN_DB)
	wcu = cx.cursor()
        pass
    
    cmd = 'select 1 from docs where doc_id = ? and published = 1'
    wcu.execute(cmd, (doc_id,))
    rows = wcu.fetchall()
    
    if cu == None:
        cx.commit()
        pass

    if len(rows) != 1:
        return False
    return True


def get_afkeys_doc(doc_id, cu=None):
    global cx
    wcu = cu
    if wcu == None:
	if not cx:
	    cx = sqlite3.connect(GINGIN_DB)
	wcu = cx.cursor()
        pass
    
    cmd = 'select afkey from afiles where doc_id = ?'
    wcu.execute(cmd, (doc_id,))
    rows = wcu.fetchall()
    afkeys = map(lambda x: x[0], rows)
    
    return afkeys


def get_hot_keywords(hot_n, cu=None):
    global cx
    wcu = cu
    if wcu == None:
	if not cx:
	    cx = sqlite3.connect(GINGIN_DB)
	wcu = cx.cursor()
    
    cmd = 'select * from hot_kws where num > 0 order by num desc limit ?'
    wcu.execute(cmd, (hot_n,))
    rows = wcu.fetchall()
    kws = map(lambda x: (x[0], x[1]), rows);
    
    return kws

def trackback_ping(doc_id, title, excerpt, url, blog_name, rhost, cu=None):
    global cx
    wcu = cu
    if wcu == None:
	if not cx:
	    cx = sqlite3.connect(GINGIN_DB)
	wcu = cx.cursor()
    
    cmd = 'insert into trackbacks values(?, ?, ?, ?, ?, ?, ?)'
    wcu.execute(cmd, (doc_id, title, excerpt, url, blog_name, rhost, now()))
    if cu == None:
	cx.commit()
    return

def get_trackback_pings(doc_id, cu=None):
    global cx
    wcu = cu
    if wcu == None:
	if not cx:
	    cx = sqlite3.connect(GINGIN_DB)
	wcu = cx.cursor()
    
    cmd = 'select * from trackbacks where doc_id=?'
    wcu.execute(cmd, (doc_id,))
    rows = wcu.fetchall()
    pings = map(lambda x: (x[1], x[2], x[3], x[4], x[5], x[6]), rows)
    if cu == None:
	cx.commit()
    return pings

def get_referers_of_doc(doc_id, cu=None):
    global cx
    wcu = cu
    if wcu == None:
	if not cx:
	    cx = sqlite3.connect(GINGIN_DB)
	wcu = cx.cursor()
    cmd = 'select src_url from referers where doc_id=?'
    wcu.execute(cmd, (doc_id,))
    rows = wcu.fetchall()
    referers = [x[0] for x in rows]
    if cu == None:
	cx.commit()
    return referers

def add_referer(doc_id, url, cu=None):
    global cx
    wcu = cu
    if wcu == None:
	if not cx:
	    cx = sqlite3.connect(GINGIN_DB)
	wcu = cx.cursor()
    cmd = 'insert into referers select max(ref_id)+1, ?, ? from referers'
    try:
	wcu.execute(cmd, (doc_id, url))
    except sqlite3.DatabaseError:
	pass

    if cu == None:
	cx.commit()
    pass



#!python
import imp
import sys
import os
from os import path
from shutil import copytree, copy
import sqlite3

try:
    pkg_dir = imp.find_module('GinGin')[1]
except:
    printf >>sys.stderr, 'Can not locate module \'GinGin\'.'
    sys.exit(1)
    pass

data_dir = path.join(pkg_dir, 'data')

js_dir = path.join(data_dir, 'js')
copytree(js_dir, 'js')

images_dir = path.join(data_dir, 'images')
copytree(images_dir, 'images')

def cpdatafile(src, dst=None):
    fn = path.join(data_dir, src)
    if not dst:
        dst = src
        pass
    copy(fn, dst)
    pass

cpdatafile(path.join('css', 'GinGin_classic.css'), 'GinGin.css')
cpdatafile('GinGin_CGI.py', 'GinGin_CGI.py')
cpdatafile('GinGin_CGI.py', 'GinGin_CGI_s.py')
cpdatafile('GinGin_CGI.py', 'GinGin_TB.py')
cpdatafile('GinGin_show_doc_src.txt',)
cpdatafile('add_url.html')
cpdatafile('dot.htaccess', '.htaccess')
os.mkdir('attaches')
cpdatafile('attaches-dot.htaccess', path.join('attaches', '.htaccess'))
file('passwd', 'w+')
cpdatafile('config.py.example', 'config.py')

sql_sch = path.join(data_dir, 'GinGin.sql')
sql_sch_fo = file(sql_sch, 'r')
sql_cmds = sql_sch_fo.read()
sql_sch_fo.close()
cx = sqlite3.connect('GinGin.db')
cu = cx.cursor()
for cmd in sql_cmds.split(';\n\n'):
    cu.execute(cmd.lstrip())
    pass
cx.commit()
cx.close()

print '''
Please make sure your WEB server running with permissions to
 1. execute scripts in this directory (GinGin_CGI.py and GinGin_TB.py),
 2. read and write database 'GinGin.db',
 3. read and write this directory '%s', and
 4. read and write subdirctory 'attaches/'.
''' % (os.getcwd())




# -*-: big5
from mez_xml.tools import nez_web

class edit_url(nez_web):
    def __init__(self, ofo):
        super(edit_url, self).__init__()
        self.ofo = ofo
        pass
    
    def _root(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<div id="content">

<div class="title">
新增/編輯 URL
</div>

<div class="text">
<form action="''')
            odata = data.setdefault('base_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''GinGin_CGI.py" method="post">
<input type="hidden" name="func" value="update_url"></input>
<input type="hidden" name="url_id" value="''')
            odata = data.setdefault('url_id', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''"></input>
<table>
<tr>
 <td>標題</td>
 <td><input size="40" type="text" name="title" value="''')
            odata = data.setdefault('url_title', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''" maxlength="80"></input></td>
</tr>

<tr>
 <td>URL</td><td>
 <input size="40" type="text" name="url" value="''')
            odata = data.setdefault('url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''" maxlength="160"></input></td>
</tr>

<tr>
 <td>摘要</td>
 <td><textarea rows="5" name="abstract" cols="40">''')
            odata = data.setdefault('abstract', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</textarea></td>
</tr>

<tr>
 <td>關鍵字</td>
 <td>
<input size="40" maxlength="200" type="text" id="keywords" name="keywords"></input>
<div id="kw_store">
''')
            cdata = data.setdefault('keywords', {})
            self.keywords(data, cdata)
            self.ofo.write('''
</div>
<script type="text/javascript" language="javascript">
<!--
function show_keyword() {
	var kws_o = document.getElementById("keywords");
	var kw_store = document.getElementById("kw_store");
	var span_o;
	var kws = [];
	var i;

	for(i = 0; i < kw_store.childNodes.length; i++) {
		if(kw_store.childNodes[i].tagName == "span" || kw_store.childNodes[i].tagName == "SPAN") {
			span_o = kw_store.childNodes[i];
			kws.push(span_o.childNodes[0].data);
		}
	}
	kws_o.setAttribute("value", kws.join("; "));
	kw_store.parentNode.removeChild(kw_store);
}
init_funcs.push(show_keyword);
-->
</script>
 </td>
</tr>
</table>

<input type="submit" value="確定"></input>
</form>
* 以此 script 新增 URL:
<a href="javascript: void(window.open("''')
            odata = data.setdefault('base_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''add_url.html?title=" + escape(document.title) + "&url=" + escape(document.URL), "gingin_url", "toolbar=no,menubar=no,width=500,height=300"))">
Bookmark</a>
</div>

</div>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    def keywords(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<span ezid="keywords">''')
            odata = data.setdefault('keyword', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</span>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    pass




# -*-: big5
from mez_xml.tools import nez_web

class attaches(nez_web):
    def __init__(self, ofo):
        super(attaches, self).__init__()
        self.ofo = ofo
        pass
    
    def _root(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<div id="content">

<div class="title">附加資料
<a href="''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/show_id_doc/''')
            odata = data.setdefault('doc_id', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''">
''')
            odata = data.setdefault('doc_id', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''
</a>
號文章
</div>
<table>
''')
            cdata = data.setdefault('attaches', {})
            self.attaches(data, cdata)
            self.ofo.write('''

<tr><td>共 ''')
            odata = data.setdefault('num_attaches', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write(''' 筆</td><td> </td></tr>

</table>
<br></br>
<br></br>
<form action="''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''" method="post" enctype="multipart/form-data">
<input type="hidden" name="func" value="add_afile"></input>
<input type="hidden" id="add_doc_id" value="''')
            odata = data.setdefault('doc_id', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''" name="doc_id"></input>
<div class="title">新增附加資料</div>
<table>

<tr><td>索引字:</td>
<td><input type="text" name="afkey"></input></td></tr>
<tr><td>檔案:</td>

<td>
<input type="file" name="afile"></input>
</td></tr>
<tr><td> </td>
<td>
<input type="submit" value="上傳"></input>
</td></tr>
</table>
</form>

<form action="''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''" method="post" id="del_form">
<input type="hidden" name="func" value="del_afile"></input>
<input type="hidden" id="del_doc_id" value="''')
            odata = data.setdefault('doc_id', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''" name="doc_id"></input>
<input type="hidden" id="del_afkey" name="afkey"></input>
</form>

</div>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    def attaches(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<tr ezid="attaches"><td>
<a href="''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/get_afile/''')
            odata = data.setdefault('doc_id', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''/''')
            odata = data.setdefault('afkey', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''">
''')
            odata = data.setdefault('afkey', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''
</a>
</td><td><input type="button" value="刪除" onclick="del_afile(\'''')
            odata = data.setdefault('afkey', {})
            self.ofo.write(self._esc_param(odata))
            self.ofo.write('''')"></input></td></tr>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    pass




from distutils.core import setup

charset = 'big5'

class fake_file(object):
    pass

def trans_templates():
    import os
    from os import path
    try:
        from mez_xml import mez_xml
    except:
        import sys
        print >>sys.stderr, 'Please install mez_xml before running setup.py.'
        sys.exit(1)
        pass
    
    temp_dir = path.join('temp_src', charset)
    fns = [fn for fn in os.listdir(temp_dir) if fn.endswith('.temp')]
    for fn in fns:
        temp = path.join(temp_dir, fn)
        pytemp = path.join('GinGin', 'templates', fn[:-5] + '.py')
                                        # s/.temp$/.py/
        ifo = file(temp, 'r')
        ofo = file(pytemp, 'w+')
        fake_ifo = fake_file()
        fake_ifo.read = lambda x=None: ifo.read(x).decode(charset).encode('utf8')
        fake_ofo = fake_file()
        fake_ofo.write = lambda x: ofo.write(x.encode(charset))
        print >>ofo, '# -*-: %s' % (charset)
        mx = mez_xml.mez_xml()
        mx.start(fn, fake_ifo, fake_ofo)
        pass
    pass

trans_templates()

setup(name='GinGin',
      version='1.0.0',
      packages=['GinGin', 'GinGin.templates'],
      package_data={'GinGin': ['data/GinGin.sql',
                               'data/GinGin_CGI.py',
                               'data/GinGin_show_doc_src.txt',
                               'data/add_url.html',
                               'data/dot.htaccess',
                               'data/attaches-dot.htaccess',
                               'data/config.py.example',
                               'data/css/*.css',
                               'data/images/GinGin.svg',
                               'data/images/*/*.jpg',
                               'data/js/*.js']},
      scripts=['scripts/GinGin_host.py'],
      requires=['mez_xml', '_sqlite3', 'fcgi'],
      author='Thinker K.F. Li',
      author_email='thinker@branda.to',
      url='http://trac-hg.assembla.com/GinGin/',
      description='GinGin is a hybrid of WIKI and BLOG system.',
      long_description='''
GinGin is a hybrid of WIKI and BLOG.  Users of GinGin can setup tags for
their documents.  The documents will be hyper-linked, automatically, when
tags are in the content of them.  In another word, GinGin will create
cross-reference for documents by the tags specified by users.

GinGin also has capability to tag URLs.  URLs are associated with tags
and cross-referenced for documents.
      ''',
      classifiers=['Programming Language :: Python',
                   'License :: OSI Approved :: BSD License',
                   'Natural Language :: Chinese (Traditional)',
                   'Topic :: Communications'],
      )



# -*-: big5
from mez_xml.tools import nez_web

class tb_ping_err(nez_web):
    def __init__(self, ofo):
        super(tb_ping_err, self).__init__()
        self.ofo = ofo
        pass
    
    def _root(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''''')
            cdata = data.setdefault('content', {})
            self.content(data, cdata)
            self.ofo.write('''''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    def content(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<response ezid="content">
<error>''')
            odata = data.setdefault('ecode', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</error>
''')
            cdata = data.setdefault('msg', {})
            self.msg(data, cdata)
            self.ofo.write('''
</response>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    def msg(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<message ezid="msg">''')
            odata = data.setdefault('data', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</message>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    pass




import os
from config import *
import GinGin_db
import represent
import types
import tools
from string import split

def mixin(cls, mixcls):
    cls.__bases__ += (mixcls,)
    pass

def get_keywords(str):
    kws = split(str, ';')
    rkws = []
    for kw in kws:
	while len(kw):
	    if kw[0] == ' ':
		kw = kw[1:]
	    elif kw[-1] == ' ':
		kw = kw[:-1]
	    else:
		break
	if len(kw):
	    rkws.append(kw)
    return rkws


def get_url_of_doc(doc_id, env):
    link = 'http://' + env['SERVER_NAME']
    if env['SERVER_PORT'] != '80':
	link = link + ':80'
    link = link + env['SCRIPT_NAME']
    last = link.rfind('/')
    link = link[:last + 1] + 'GinGin_CGI.py'
    link = link + '/show_id_doc/' + str(doc_id)
    return link


def gingin_date_to_http_date(tm):
    from time import asctime, strptime
    return asctime(strptime(tm, '%Y-%m-%d %H:%M:%S %Z'))


def gingin_date_to_rss_date(tm):
    from time import strftime, strptime, timezone
    zone = timezone
    if timezone < 0:
	zone = -zone
	sig = '-'
    else:
	sig = '+'
	pass
    zh = zone/(60*60)
    zm = (zone / 60) % 60
    zonestr = '%s%02d:%02d' % (sig, zh, zm)
    tm = strptime(tm, '%Y-%m-%d %H:%M:%S %Z')
    return strftime('%Y-%m-%dT%H:%M:%S' + zonestr, tm)


class EMPTY_ACTION:
    def __init__(self, msg):
	self.__msg = msg
	
    def __repr__(self):
	return "EMPTY_ACTION: " + self.__msg
    
    def __str__(self):
	return "EMPTY_ACTION: " + self.__msg
    pass


class DENY_ACTION:
    def __init__(self, user, act_name, doc_id):
	self.user = user
	self.act_name = act_name
	self.doc_id = doc_id
	pass
    pass

# ========================================

## \defgroup app_action_G Application Actions
#
# Every request from user client will trigger some actions to serve
# the user.  An action can as complex as performing a transaction, or
# as easy as reading a file.  All these actions are modeled as
# \ref new_action classes.  Every request will trigger a corresponding
# new_action class.  The class would be instantiated and called; it is
# a callable.
#
# \ref action_form_G are used to be mixed-in with \ref new_action to
# provide form variables. These variables are parameters provided by
# users.  new_action objects read variables and customize actions for
# users.
#
# \ref new_octr_G controls output stream of new_action objects.
# new_action objects can specify Content-Type or other headers
# and tell \ref new_octr_G how to generate the output from data provided by
# new_action objects.

## An new_action is a callable that performs application actions.
# \ingroup app_action_G
#
# New_action is abstraction of actions of GinGin.
# Every WEB requestion from user client triggers a new_action
# to perform actions.
#
# A new_action is a callable that being called to perform actions.
class new_action(object):
    def __init__(self, act_name, ival=[], fval=[]):
	super(new_action, self).__init__()
	self._act_name = act_name
	self._ival = ival
        self._fval = fval
	self.encoding = GINGIN_ENCODING
	pass

    ## Authorize the user to access a document.
    #
    # \param doc_id is ID of the document be authorized.
    # \return User ID.
    #
    # If doc_id is -1, it is authorized for an action
    # that is not for any document.
    def auth(self, doc_id=-1):
	user = GINGIN_USER.get_login_user(self.octr)
	if not GINGIN_USER.authorize(self, user, self._act_name, doc_id):
	    raise DENY_ACTION(user, self._act_name, doc_id)
	return user

    ## Conventional function for transcoding locale string to UTF8.
    def to_utf8(self, data, enc=None):
	if enc == None:
	    enc = self.encoding
	return data.decode(enc).encode('utf8')

    ## Execute the action.
    def __call__(self):
	raise NotImplementedError, '%s.__call__()' % (self.__class__.__name__,)
    pass

## \defgroup action_form_G Action Form
# \ingroup app_action_G
# \brief Action_form initialize form variables provided by user client.
#@{

## Setup form variables.
#
# It sets self.form to be a dictionary for an action.  The dictinary
# is a respository of variables from user clients.  It also sers
# self.env to deliver environment variables from HTTP daemons.
class action_form(object):
    def __init__(self, *args, **kws):
	super(action_form, self).__init__(*args, **kws)
	pass
    
    #form = mkundef('form')
    pass


## Action_form for CGI environment.
#
# It reads variables from PATH_INFO and CGI form variables.
# By definition of http://hoohoo.ncsa.uiuc.edu/cgi/env.html,
# PATH_INFO is
# \verbatim
#       The extra path information, as given by the client.
#       In other words, scripts can be accessed by their virtual
#       pathname, followed by extra information at the end of
#       this path.
# \endverbatim
class CGI_form(action_form):
    def __init__(self, *args, **kws):
	super(CGI_form, self).__init__(*args, **kws)
	pass
    
    def __full_with_CGI_form(self, res, form):
	for var in self._ival:
	    try:
                res[var] = self.to_utf8(form.getvalue(var))
                pass
            except:
		pass
	    pass
        # Upload files
	for var in self._fval:
	    try:
                res[var] = form[var]
                pass
            except:
		pass
	    pass
	pass
    
    def __full_with_path_info(self, res):
	encoding = GINGIN_ENCODING
	try:
	    if self.env['HTTP_USER_AGENT'].find(' MSIE ') >= 0:
		encoding = 'utf8'
	except:
	    pass
	try:
	    pinfo = split(self.env['PATH_INFO'], '/')[2:]
	    i = 0
	    while i < len(pinfo) and i < len(self._ival):
		var = self._ival[i]
		res[var] = self.to_utf8(pinfo[i], encoding)
		i = i + 1
	except KeyError:
	    pass
	pass

    ## Setup form/env variables from variables of CGI form and PATH_INFO.
    def set_form(self, form, env):
        self.env = env
	self.form = {}
	self.__full_with_CGI_form(self.form, form)
	self.__full_with_path_info(self.form)
	pass
    pass


## A mock of action_form.
#
# It is used for (unit) testing.
class test_form(action_form):
    def __init__(self, *args, **kws):
	super(test_form, self).__init__(*args, **kws)
	pass
    
    def set_form(self, form, env):
	self.form = form
        self.env = env
	pass
    pass

##@}

## \defgroup new_octr_G Output Control
# \ingroup app_action_G
# \brief Control and format output of actions.
#
# \ref new_octr control and format output of actions (\ref new_action).
# \ref action_octr is responsive to setup a new_octr for a new_action.
#@{

## Control ouptput of actions.
class new_octr(object):
    def __init__(self, *args, **kws):
        super(new_octr, self).__init__(*args, **kws)
        self.media_type = 'text/html'
        self.charset = GINGIN_ENCODING
        pass

    ## Setup media type (Content-Type) of output page.
    def set_media_type(self, mtype):
        self.media_type = mtype
        pass

    ## Setup charset of output page.
    def set_charset(self, chset):
        self.charset = chset
        pass
    
    #ofo = mkundef('ofo')
    pass

## Initialize self.octr as a new_octr.
class action_octr(object):
    def __init__(self, *args, **kws):
	super(action_octr, self).__init__(*args, **kws)
	pass
    
    #octr = mkundef('octr')
    pass


## Action_octr for CGI environment.
class CGI_octr(action_octr):
    def __init__(self, *args, **kws):
	super(CGI_octr, self).__init__(*args, **kws)
	pass
    
    def set_octr(self, octr):
	self.octr = octr
	pass
    pass

## A mock of new_octr.
#
# It is used for (unit) testing.
class test_octr(action_octr):
    def __init__(self, *args, **kws):
	import sys
	super(test_octr, self).__init__(*args, **kws)
	self.octr = new_octr()
	self.octr.ofo = sys.stdout
	pass
    octr = 1
    pass

## @}

# ========================================

## \defgroup actions_G GinGin's actions.
# \ingroup app_action_G
# @{

class default_act(new_action):
    def __init__(self):
        super(default_act, self).__init__('default_act',  ['func'])
        pass
    
    def __call__(self):
        form = self.form
        try:
            func = form['func']
        except:
            try:
                func = split(self.env['PATH_INFO'], '/')[1]
            except:
                func = ''
                pass
            pass
        return represent.error_page('no such function: %s' % (func),
                                    self.octr.ofo)
    pass

## Show the document with specified ID.
class show_id_doc(new_action):
    def __init__(self):
	super(show_id_doc, self).__init__('show_id_doc',  ['doc_id'])
	pass
	
    def __call__(self):
	from templates import doc
	from xml_template import escape_url

	doc_id = int(self.form['doc_id'])

        self.auth(doc_id)
	
	kws, doc_id, title, doc_body, pdate, uid, published = \
            GinGin_db.get_id_doc(doc_id)
	content = {}
	creator = GINGIN_USER.get_user_from_id(self, uid)
	all_kws = GinGin_db.get_all_kws()
	doc_body = GinGin_db.replace_kws(doc_body, all_kws)
	
	comments = GinGin_db.get_id_doc_comments(doc_id)
	comments = [{'msg': x[4],
                     'website': escape_url(x[3]),
                     'pdate': x[5],
                     'name': x[1]}
                    for x in comments]
	
	content['keywords'] = [{'keyword': kw} for kw in kws]
	content['doc_id'] = doc_id
	content['title'] = title
	content['text'] = doc_body
	content['pdate'] = pdate
	content['uid'] = uid
	content['creator'] = creator
	content['comment'] = comments
	
	doc.str = tools.hack_str
	
	return content, doc.doc(self.octr.ofo)
    pass


## Show the source of the document of specified ID.
class show_id_doc_src(new_action):
    def __init__(self):
	super(show_id_doc_src, self).__init__('show_id_doc_src',  ['doc_id'])
	
    def __call__(self):
	doc_id = int(self.form['doc_id'])

        self.auth(doc_id)

	kws, doc_id, title, doc_body, pdate, uid, published = \
            GinGin_db.get_id_doc(doc_id)
	creator = GINGIN_USER.get_user_from_id(self, uid)
	represent.show_doc_src(kws, doc_id, title, doc_body, pdate, creator)
	pass
    pass

	
## Show editor form for the document with specified ID.
class edit_id_doc(new_action):
    def __init__(self):
	super(edit_id_doc, self).__init__('edit_id_doc', ['doc_id'])
	pass
	
    def __call__(self):
	from templates import edit
	doc_id = int(self.form['doc_id'])
        self.auth(doc_id)
	
	kws, doc_id, title, doc_body, pdate, uid, published = \
            GinGin_db.get_id_doc(doc_id)
	
	content = {}
	content['keywords'] = [{'keyword': kw} for kw in kws]
	content['doc_id'] = doc_id
	content['title'] = title
	content['body'] = doc_body
        content['published'] = published and 'true' or 'false'
	
	edit.str = tools.hack_str
	
	return content, edit.edit(self.octr.ofo)
    pass


## Show unpublished docs.
class show_all_unpublished(new_action):
    def __init__(self):
        super(show_all_unpublished, self).__init__('show_all_unpublished', [])
        pass

    def __call__(self):
        from templates import unpublished

        user = self.auth()

        docs = GinGin_db.get_unpublished_docs()
        docs = [{'doc_id': doc[0], 'title': doc[1],
                 'pdate': doc[2], 'creator': doc[3]}
                for doc in docs]
        content = {'title': 'List of unpublished documents', 'docs': docs}
        return content, unpublished.unpublished(self.octr.ofo)
    pass


## Show all keywords of a GinGin system.
class show_all_kws(new_action):
    def __init__(self):
	super(show_all_kws, self).__init__('show_all_kws', [])
	pass
    
    def __call__(self):
	from templates import all_kws
	
	kws = GinGin_db.get_all_kws()
	kws = [{'keyword': kw} for kw in kws]
	content = {'title': 'All Keywords', 'keywords': kws, 'keywords_num': len(kws)}
	
	all_kws.str = tools.hack_str
	
	return content, all_kws.all_kws(self.octr.ofo)
    pass


## Update content of the document specified ID.
class update(new_action):
    def __init__(self):
	super(update, self).__init__('update',
                                         ['doc_id', 'title', 'keywords',
                                          'doc_body', 'update_pdate',
                                          'published'])
        pass
    
    def __call__(self):
        from tools import get_server_url

        form = self.form
	doc_id = int(form['doc_id'])
	user = self.auth(doc_id)
	
	try:
	    title = form['title']
	except:
	    return represent.error_page('TITLE is need!', self.octr.ofo)
	try:
	    kws = get_keywords(form['keywords'])
	except:
	    return represent.error_page('At least one keyword!',
                                        self.octr.ofo)
	try:
	    doc_body = form['doc_body']
	except:
	    doc_body = ''
	    pass
	
	try:
	    update_pdate = form['update_pdate']
	except:
	    update_pdate = None
	    pass
	if update_pdate == 'yes':
	    update_pdate = True
	    pass

        try:
            published = form['published']
        except:
            published = None
            pass
        if published == 'yes':
            published = True
            pass
	
	if len(doc_body) > MAX_DOC_SZ:
	    return represent.error_page('too long', self.octr.ofo)
        uid = GINGIN_USER.get_id_from_user(self, user)
        doc_id = GinGin_db.update_doc(doc_id, title, kws, doc_body,
                                      uid, update_pdate, published)
        base, spath = get_server_url(self.octr.env)
        represent.show_update_ok(doc_id, title, spath)
        return None

## Show documents and URLs of a keyword.
class show_kw_docs(new_action):
    def __init__(self):
	super(show_kw_docs, self).__init__('show_kw_docs', ['keyword'])
	pass
    
    def __call__(self):
	from templates import keyword
	from xml_template import escape_url
	
	kw = self.form['keyword'].decode('utf-8')
	docs = GinGin_db.get_kw_docs(kw)
	urls = GinGin_db.get_kw_urls(kw)
	if len(docs) > 1 or len(urls) > 0:
	    docs =[{'doc_id': d[0], 'title': d[1], 'pdate': d[2], 'creator': d[3]} for d in docs]
	    urls = [{'url_id': u[0], 'title': u[1], 'url': escape_url(u[2]), 'abstract': u[3], 'pdate': u[4], 'creator': u[5]} for u in urls]
	    content = {'name': kw, 'docs': docs, 'urls': urls, 'title': kw}
	    keyword.str = tools.hack_str
	    return content, keyword.keyword(self.octr.ofo)
	elif len(docs) == 1:
	    # why not redirect ?
	    base, spath = tools.get_server_url(self.env)
	    tgt = spath + '/show_id_doc/%d'
	    tools.redirect(self.octr.ofo, tgt % (docs[0][0],))
	else:
	    return represent.error_page('%s is not a keyword' % (kw),
                                        self.octr.ofo)
	pass
    pass

## Show a form for a new document.
class new_doc(new_action):
    def __init__(self):
	super(new_doc, self).__init__('new_doc', [])
	pass
	
    def __call__(self):
	from templates import edit
	content = {'keywords': [], 'doc_id': -1, 'title': '', 'body': ''}
	
	edit.str = tools.hack_str
	
	return content, edit.edit(self.octr.ofo)

## Upload an attachment.
class add_afile(new_action):
    def __init__(self):
        super(add_afile, self).__init__('add_afile',
                                        ['doc_id', 'afkey'],
                                        ['afile'])
        pass
	
    def __call__(self):
        form = self.form
	doc_id = int(form['doc_id'])
	self.auth(doc_id)
	
	afkey = form['afkey']
	afobj = form['afile'].file
	mtype = form['afile'].type
	GinGin_db.add_afile_to_doc(doc_id, afkey, afobj, mtype)
	represent.go_back(self.env)
        pass

## Delete an attachment.
class del_afile(new_action):
    def __init__(self):
	super(del_afile, self).__init__('del_afile', ['doc_id', 'afkey'])
        pass
	
    def __call__(self):
        form = self.form
	doc_id = int(form['doc_id'])
	self.auth(doc_id)
	
	afkey = form['afkey']
	GinGin_db.remove_afile_by_afkey(doc_id, afkey)
	represent.go_back(self.env)
        pass
	
## Get content of an attachement.
class get_afile(new_action):
    def __init__(self):
	super(get_afile, self).__init__('get_afile', ['doc_id', 'afkey'])
	
    def __call__(self):
        form = self.form
	doc_id = int(form['doc_id'])
        self.auth(doc_id)
	
	afkey = form['afkey']
	afname, mtype = \
            GinGin_db.get_afname_mtype_by_afkey(doc_id, afkey)
	if afname != None:
	    represent.send_file(afname, afkey, mtype)
	else:
	    represent.not_found(afkey)
            pass
        pass
    pass

## Show attachments of the document with specified ID.
class show_afiles(new_action):
    def __init__(self):
	super(show_afiles, self).__init__('show_afiles', ['doc_id'])
        pass
	
    def __call__(self):
        from templates import attaches
        
        form = self.form
	doc_id = int(form['doc_id'])
        self.auth(doc_id)

	afkeys = GinGin_db.get_afkeys_doc(doc_id)
        content = {'doc_id': doc_id,
                   'title': 'Attachments of doc #%d' % (doc_id)}
        _attaches = [{'afkey': afkey} for afkey in afkeys]
        content['attaches'] = _attaches
        content['num_attaches'] = len(_attaches)

        attaches.str = tools.hack_str
        return content, attaches.attaches(self.octr.ofo)
    pass

## Show URLs with specified keyword.
class show_kw_urls(new_action):
    def __init__(self):
	super(show_kw_urls, self).__init__('show_kw_urls', ['keyword'])
	pass
	
    def __call__(self):
	from templates import kw_urls
	form = self.form

	keyword = form['keyword'].decode('utf8')
	urls = GinGin_db.get_kw_urls(keyword)
	content = represent.show_kw_urls(keyword, urls)
	content['title'] = keyword
	
	kw_urls.str = tools.hack_str
	
	return content, kw_urls.kw_urls(self.octr.ofo)

## Unassociate URLs with specified keyword.
class del_kw_urls(new_action):
    def __init__(self):
	super(del_kw_urls, self).__init__('del_kw_urls', ['keyword', 'url_id'])
	
    def __call__(self):
	import types
	
	self.auth()
	form = self.form
	
        keyword = form['keyword']
        if form.has_key('url_id'):
            url_ids = form['url_id']
            if isinstance(url_ids, types.StringTypes):
                url_ids = [url_ids]
                pass
            url_ids = map(lambda x: int(x), url_ids)
            GinGin_db.del_kw_urls(keyword, url_ids)
        else:
            url_ids = []
            pass
	represent.show_del_kw_urls_ok(keyword, url_ids)

## Show document links for spider.
class show_to_robot(new_action):
    def __init__(self):
	super(show_to_robot, self).__init__('show_to_robot', [])
	
    def __call__(self):
	docs = GinGin_db.get_last_doc_ids(GINGIN_TO_ROBOT_N)
	represent.show_to_robot(docs, self.env)
	pass
    pass


## Return RSS feed of the GinGin system.
class rssfeed(new_action):
    def __init__(self):
	super(rssfeed, self).__init__('rssfeed', [])
	pass
    
    def __call__(self):
	from templates import rssfeed
	
	docs = GinGin_db.get_last_abstracts(GINGIN_TO_RSSFEED_N)
	docs = map(lambda x: (x[0], x[1], gingin_date_to_rss_date(x[2]), x[3], x[4]), docs)
	self.octr.set_media_type('application/xml')
	content = represent.show_rssfeed(docs)
	content['title'] = GINGIN_SITE
	
	rssfeed.str = tools.hack_str
	
	return content, rssfeed.rssfeed(self.octr.ofo), None
    pass

## Update/add an URL.
class update_url(new_action):
    def __init__(self):
	super(update_url, self).__init__('update_url', ['keywords', 'url_id', 'title', 'url', 'abstract'])
	pass
	
    def __call__(self):
	from templates import edit_url

	user = self.auth()
        
	try:
	    abstract = self.form['abstract']
	except KeyError:
	    abstract = ''
	try:
	    url_id = int(self.form['url_id'])
	    title = self.form['title']
	    url = self.form['url']
	    kws = self.form['keywords']
	    kws = get_keywords(kws)
	    if len(kws) == 0:
		return represent.error_page(
                    'you should specify at leat one keyword.',
                    self.octr.ofo)
            pass
        except KeyError:
	    return represent.error_page('paramter error', self.octr.ofo)
	uid = GINGIN_USER.get_id_from_user(self, user)
	url_id = GinGin_db.update_url(url_id, title, url, abstract, uid, kws)
	base, spath = tools.get_server_url(self.env)
	tools.redirect(self.octr.ofo, '%s/show_kw_urls/%s' % (spath, str(kws[0].decode('utf-8'))))
	pass
    pass


## Show a form for adding a new URL.
class edit_new_url(new_action):
    def __init__(self):
	super(edit_new_url, self).__init__('edit_new_url', [])
	pass
	
    def __call__(self):
	from templates import edit_url
	
	content = {'title': 'Edit a URL', 'url_id': -1, 'url_title': '', 'url': '', 'abstract': '', 'keywords': []}
	
	edit_url.str = tools.hack_str
	
	return content, edit_url.edit_url(self.octr.ofo)
    pass


## Show a form for editing an exisited URL.
class edit_url(new_action):
    def __init__(self):
	super(edit_url, self).__init__('edit_url', ['url_id'])
	pass
    
    def __call__(self):
	from templates import edit_url
	
	url_id = int(self.form['url_id'])
	url, kws = GinGin_db.get_url(url_id)
	
	content = {'title': 'Edit a URL',
                   'url_id': url_id,
                   'url_title': url[1],
                   'url': url[2],
                   'abstract': url[3]}
	content['keywords'] = [{'keyword': kw} for kw in kws]
	
	edit_url.str = tools.hack_str
	
	return content, edit_url.edit_url(self.octr.ofo)
    pass

## Show a form to create a new comment for the document with specified ID.
class edit_comment(new_action):
    def __init__(self):
	super(edit_comment, self).__init__('edit_comment', ['doc_id'])
	pass
    
    def __call__(self):
	from templates import comment
	
	doc_id = int(self.form['doc_id'])
	attack = GinGin_db.get_id_doc_attack(doc_id)
	content = {'title': 'Edit Comment', 'doc_id': doc_id, 'attack_plain': attack[1], 'attack_cipher': attack[2]}
	
	comment.str = tools.hack_str
	
	return content, comment.comment(self.octr.ofo)
    pass

## Add a comment to the document with specified ID.
class add_comment(new_action):
    def __init__(self):
	super(add_comment, self).__init__('add_comment', ['doc_id', 'key', 'name', 'email', 'website', 'msg'])
	pass
    
    def __call__(self):
        from tools import get_server_url

	form = self.form
	
	doc_id = int(form['doc_id'])
	key = form['key']
	name = form['name']
	email = form['email']
	if form.has_key('website'):
	    website = form['website']
	else:
	    website = None
	    pass
	msg = form['msg']
	GinGin_db.add_id_doc_comment(doc_id, name, email, website, msg, key)
	
        base, spath = get_server_url(self.octr.env)
        represent.show_update_ok(doc_id, '', spath)
	pass
    pass

## Show lastest comments of the GinGin system.
class last_comments(new_action):
    def __init__(self):
	super(last_comments, self).__init__('last_comments', [])
	pass
    
    def __call__(self):
	from templates import last_comments
	comments = GinGin_db.get_last_comments(GINGIN_LAST_COMMENTS_N)
	cmts = [{'cmt_id': c[0], 'doc_id': c[1], 'name': c[2], 'website': c[4], 'msg': c[5], 'pdate': c[6]} for c in comments]
	content = {'title': 'Last Comments', 'comments': cmts}
	
	last_comments.str = tools.hack_str
	
	return content, last_comments.last_comments(self.octr.ofo)

## Show trackback of a specified document.
class trackback(new_action):
    def __init__(self):
	super(trackback, self).__init__('trackback', ['doc_id', '__mod'])
        pass

    def __call__(self):
        form = self.form
	doc_id = int(form['doc_id'])
	
        self.auth(doc_id)

	if form.has_key('__mod'):
	    __mod = form['__mod']
	    if __mod in ('rss', 'view'):
		kws, doc_id, title, doc_body, pdate, uid, published = \
                    GinGin_db.get_id_doc(doc_id)
		desc = doc_body
		if len(desc) > GINGIN_TB_BODY_MAX:
		    desc = desc[:GINGIN_TB_BODY_MAX] + '...'
		link = get_url_of_doc(doc_id, self.env)
		pings = GinGin_db.get_trackback_pings(doc_id)
		if __mod == 'rss':
                    from templates import tb_rss
                    content = {'title': title,
                               'link': link,
                               'description': desc,
                               'language': GINGIN_LANG}
                    pings_ent = [{'title': ping[0],
                                  'description': ping[2],
                                  'link': ping[3]}
                                 for ping in pings]
                    content['items'] = pings_ent
                    tb_rss.str = tools.hack_str
                    self.octr.set_media_type('text/xml')
                    return content, tb_rss.tb_rss(self.octr.ofo), None
		else:
                    from templates import tb_view
                    tb_view.str = tools.hack_str
                    content = {'title': title,
                               'doc_id': doc_id,
                               'desc': desc,
                               'link': link}
                    pings_ent = [{'title': ping[1], 'url': ping[3]}
                                 for ping in pings]
                    content['pings'] = pings_ent
                    content['num_pings'] = len(pings_ent)
                    return content, tb_view.tb_view(self.octr.ofo), None
	    else:
		return represent.error_page('invalid mode: %s' % (__mod),
                                            self.octr.ofo)
	else:
	    try:
		if form.has_key('title'):
		    title = form['title']
		else:
		    title = ''
		    pass
		if len(title) > 128:
		    title = title[:128] + '...'
		    pass
		
		if form.has_key('excerpt'):
		    excerpt = form['excerpt']
		else:
		    excerpt = ''
		    pass
		if len(excerpt) > GINGIN_TB_BODY_MAX:
		    excerpt = excerpt[:GINGIN_TB_BODY_MAX] + '...'
		    pass
		
		url = form['url']
		if len(url) > 128:
		    url = url[:128]
		    pass
		
		if form.has_key('blog_name'):
		    blog_name = form['blog_name']
		else:
		    blog_name = ''
		    pass
		if len(url) > 128:
		    blog_name = blog[:128]
		    pass
		pass
	    except KeyError:
                emsg = 'invalid arguments'
            else:
                rhost = self.env['REMOTE_ADDR'] + ':' + self.env['REMOTE_PORT']
                if exist_doc_id(doc_id):
                    GinGin_db.trackback_ping(doc_id, title, excerpt, url, blog_name, rhost)
                    emsg = None
                else:
                    emsg = 'doc %d is not found' % (doc_id)
                    pass
                pass
            from templates import tb_ping_err
            tb_ping_err.str = tools.hack_str
            if emsg:
                msg = {'data': emsg}
                ecode = 1
            else:
                msg = []
                ecode = 0
                pass
            content = {'title': '', 'msg': msg, 'ecode': ecode}
            self.octr.set_media_type('text/xml')
            self.octr.set_charset('iso-8859-1')
            return content, tb_ping_err.tb_ping_err(self.octr.ofo), None
        pass
    pass

## @}

def full_side_bar_info(out):
    docs = GinGin_db.get_last_docs(GINGIN_MOST_RECENT_N)
    hot_kws = GinGin_db.get_hot_keywords(GINGIN_MOST_HOT_KWS)
    
    hks = map(lambda x: {'name': x[0], 'num': int(x[1])}, hot_kws)
    lds = map(lambda x: {'id': x[0], 'title': x[1]}, docs)
    out['hot_keys'] = {'key': hks}
    out['last_docs'] = {'doc': lds}
    return out


def _get_func_name(form, env):
    try:
        func = form['func'].value
    except KeyError:
        try:
            pinfo = env['PATH_INFO']
        except:
            return None
        func = split(pinfo, '/')[1]
        pass
    return func

def _fallback_homepage(form, env):
    func = 'show_id_doc'
    env = dict(env)
    env['PATH_INFO'] = '/show_id_doc/1'
    return func, form, env

def _show_error_page(form, env, octr):
    import sys
    import traceback
    tp, val, tb = sys.exc_info()
    while tb.tb_next:
        tb = tb.tb_next
        pass
    f = tb.tb_frame
    lno = traceback.tb_lineno(tb)
    emsg1 = '%s: %s\n\tin %s, L%d file \'%s\'\n' % (str(tp), str(val),
                                                    f.f_code.co_name,
                                                    lno,
                                                    f.f_code.co_filename)
    #emsg2 = 'locals: %s\nglobals: %s\n' % (repr(f.f_locals),
    #                                       repr(f.f_globals))
    return represent.error_page(emsg1, octr.ofo)

def GinGin_run_action(form, env, octr):
    import mez_xml.tools
    mez_xml.tools.str = tools.hack_str
    
    func = _get_func_name(form, env)
    if not func:
        func, form, env = _fallback_homepage(form, env)
        pass
    
    act_clazz = _all_actions_.setdefault(func, default_act)
    
    try:
        act = act_clazz()
        act.set_form(form, env)
        act.set_octr(octr)
        return act()
    except DENY_ACTION, e:
	return represent.show_deny(e.user, e.act_name, e.doc_id)
    except user.NO_LOGIN_ERR:
	GINGIN_USER.show_login(act)
    except user.REDIRECT, e:
        represent.redirect(e.url, e.msg)
    except:
        return _show_error_page(form, env, octr)
    pass

def _init_actions():
    global _all_actions_
    _all_actions_ = {}
    for name, obj in globals().items():
        if type(obj) == type and \
                obj != new_action and \
                issubclass(obj, new_action):
            mixin(obj, CGI_form)
            mixin(obj, CGI_octr)
            _all_actions_[name] = obj
            pass
        pass
    pass

_init_actions()




# -*-: big5
from mez_xml.tools import nez_web

class rssfeed(nez_web):
    def __init__(self, ofo):
        super(rssfeed, self).__init__()
        self.ofo = ofo
        pass
    
    def _root(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<rss version="2.0" xmlns:dc="http://purl.org/dc/elements/1.1/">
<channel>
	<title>''')
            odata = data.setdefault('title', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</title>
	<link>''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</link>
	<dc:language></dc:language>
	<dc:creator>''')
            odata = data.setdefault('site', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</dc:creator>
	<dc:date></dc:date>
	''')
            cdata = data.setdefault('content', {})
            cdata = cdata.setdefault('feeds', {})
            cdata = cdata.setdefault('feed', {})
            self.feed(data, cdata)
            self.ofo.write('''
</channel>
</rss>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    def feed(self, pdata, cdata):
        def temp(data):
            self.ofo.write('''<item ezid="content.feeds.feed">
		<title>''')
            odata = data.setdefault('title', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</title>
		<link>''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''/show_id_doc/''')
            odata = data.setdefault('doc_id', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</link>
		<description>''')
            odata = data.setdefault('abs', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</description>
		<guid>''')
            odata = data.setdefault('gingin_url', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''/show_id_doc/''')
            odata = data.setdefault('doc_id', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</guid>
		<dc:date>''')
            odata = data.setdefault('pdate', {})
            self.ofo.write(self._esc_text(odata))
            self.ofo.write('''</dc:date>
	</item>''')
            pass
        self._feed_subtree(temp, pdata, cdata)
        pass
    
    pass

